/* original parser id follows */
/* yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93" */
/* (use YYMAJOR/YYMINOR for ifdefs dependent on parser version) */

#define YYBYACC 1
#define YYMAJOR 2
#define YYMINOR 0
#define YYPATCH 20210808

#define YYEMPTY        (-1)
#define yyclearin      (yychar = YYEMPTY)
#define yyerrok        (yyerrflag = 0)
#define YYRECOVERING() (yyerrflag != 0)
#define YYENOMEM       (-2)
#define YYEOF          0
#define YYPREFIX "yy"

#define YYPURE 0

#line 2 "osm_qos_parser_y.y"
/*
 * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
 * Copyright (c) 2002-2008 Mellanox Technologies LTD. All rights reserved.
 * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
 * Copyright (c) 2008 HNR Consulting. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

/*
 * Abstract:
 *    Grammar of OSM QoS parser.
 *
 * Environment:
 *    Linux User Mode
 *
 * Author:
 *    Yevgeny Kliteynik, Mellanox
 */

#include <stdio.h>
#include <assert.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <opensm/osm_file_ids.h>
#define FILE_ID OSM_FILE_QOS_PARSER_Y_Y
#include <opensm/osm_opensm.h>
#include <opensm/osm_qos_policy.h>

#define OSM_QOS_POLICY_MAX_LINE_LEN         1024*10
#define OSM_QOS_POLICY_SL2VL_TABLE_LEN      IB_MAX_NUM_VLS
#define OSM_QOS_POLICY_MAX_VL_NUM           IB_MAX_NUM_VLS
#define OSM_QOS_POLICY_MAX_RATE             IB_MAX_RATE
#define OSM_QOS_POLICY_MIN_RATE             IB_MIN_RATE
#define OSM_QOS_POLICY_MAX_MTU              IB_MAX_MTU
#define OSM_QOS_POLICY_MIN_MTU              IB_MIN_MTU

typedef struct tmp_parser_struct_t_ {
    char       str[OSM_QOS_POLICY_MAX_LINE_LEN];
    uint64_t   num_pair[2];
    cl_list_t  str_list;
    cl_list_t  num_list;
    cl_list_t  num_pair_list;
} tmp_parser_struct_t;

static void __parser_tmp_struct_init();
static void __parser_tmp_struct_reset();
static void __parser_tmp_struct_destroy();

static char * __parser_strip_white(char * str);

static void __parser_str2uint64(uint64_t * p_val, char * str);

static void __parser_port_group_start();
static int __parser_port_group_end();

static void __parser_sl2vl_scope_start();
static int __parser_sl2vl_scope_end();

static void __parser_vlarb_scope_start();
static int __parser_vlarb_scope_end();

static void __parser_qos_level_start();
static int __parser_qos_level_end();

static void __parser_match_rule_start();
static int __parser_match_rule_end();

static void __parser_ulp_match_rule_start();
static int __parser_ulp_match_rule_end();

static void __pkey_rangelist2rangearr(
    cl_list_t    * p_list,
    uint64_t  ** * p_arr,
    unsigned     * p_arr_len);

static void __rangelist2rangearr(
    cl_list_t    * p_list,
    uint64_t  ** * p_arr,
    unsigned     * p_arr_len);

static void __merge_rangearr(
    uint64_t  **   range_arr_1,
    unsigned       range_len_1,
    uint64_t  **   range_arr_2,
    unsigned       range_len_2,
    uint64_t  ** * p_arr,
    unsigned     * p_arr_len );

static void __parser_add_port_to_port_map(
    cl_qmap_t   * p_map,
    osm_physp_t * p_physp);

static void __parser_add_guid_range_to_port_map(
    cl_qmap_t  * p_map,
    uint64_t  ** range_arr,
    unsigned     range_len);

static void __parser_add_pkey_range_to_port_map(
    cl_qmap_t  * p_map,
    uint64_t  ** range_arr,
    unsigned     range_len);

static void __parser_add_partition_list_to_port_map(
    cl_qmap_t  * p_map,
    cl_list_t  * p_list);

static void __parser_add_map_to_port_map(
    cl_qmap_t * p_dmap,
    cl_map_t  * p_smap);

static int __validate_pkeys(
    uint64_t ** range_arr,
    unsigned    range_len,
    boolean_t   is_ipoib);

static void __setup_simple_qos_levels();
static void __clear_simple_qos_levels();
static void __setup_ulp_match_rules();
static void __process_ulp_match_rules();
static void yyerror(const char *format, ...);

extern char * yytext;
extern int yylex (void);
extern FILE * yyin;
extern int errno;
extern void yyrestart(FILE *input_file);
int yyparse();

#define RESET_BUFFER  __parser_tmp_struct_reset()

tmp_parser_struct_t tmp_parser_struct;

int column_num;
int line_num;

osm_qos_policy_t       * p_qos_policy = NULL;
osm_qos_port_group_t   * p_current_port_group = NULL;
osm_qos_sl2vl_scope_t  * p_current_sl2vl_scope = NULL;
osm_qos_vlarb_scope_t  * p_current_vlarb_scope = NULL;
osm_qos_level_t        * p_current_qos_level = NULL;
osm_qos_match_rule_t   * p_current_qos_match_rule = NULL;
osm_log_t              * p_qos_parser_osm_log;

/* 16 Simple QoS Levels - one for each SL */
static osm_qos_level_t osm_qos_policy_simple_qos_levels[16];

/* Default Simple QoS Level */
osm_qos_level_t __default_simple_qos_level;

/*
 * List of match rules that will be generated by the
 * qos-ulp section. These rules are concatenated to
 * the end of the usual matching rules list at the
 * end of parsing.
 */
static cl_list_t __ulp_match_rules;

/***************************************************/

#line 212 "osm_qos_parser_y.c"

#if ! defined(YYSTYPE) && ! defined(YYSTYPE_IS_DECLARED)
/* Default: YYSTYPE is the semantic value type. */
typedef int YYSTYPE;
# define YYSTYPE_IS_DECLARED 1
#endif

/* compatibility with bison */
#ifdef YYPARSE_PARAM
/* compatibility with FreeBSD */
# ifdef YYPARSE_PARAM_TYPE
#  define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
# else
#  define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
# endif
#else
# define YYPARSE_DECL() yyparse(void)
#endif

/* Parameters sent to lex. */
#ifdef YYLEX_PARAM
# define YYLEX_DECL() yylex(void *YYLEX_PARAM)
# define YYLEX yylex(YYLEX_PARAM)
#else
# define YYLEX_DECL() yylex(void)
# define YYLEX yylex()
#endif

#if !(defined(yylex) || defined(YYSTATE))
int YYLEX_DECL();
#endif

/* Parameters sent to yyerror. */
#ifndef YYERROR_DECL
#define YYERROR_DECL() yyerror(const char *s)
#endif
#ifndef YYERROR_CALL
#define YYERROR_CALL(msg) yyerror(msg)
#endif

extern int YYPARSE_DECL();

#define TK_NUMBER 257
#define TK_DASH 258
#define TK_DOTDOT 259
#define TK_COMMA 260
#define TK_ASTERISK 261
#define TK_TEXT 262
#define TK_QOS_ULPS_START 263
#define TK_QOS_ULPS_END 264
#define TK_PORT_GROUPS_START 265
#define TK_PORT_GROUPS_END 266
#define TK_PORT_GROUP_START 267
#define TK_PORT_GROUP_END 268
#define TK_QOS_SETUP_START 269
#define TK_QOS_SETUP_END 270
#define TK_VLARB_TABLES_START 271
#define TK_VLARB_TABLES_END 272
#define TK_VLARB_SCOPE_START 273
#define TK_VLARB_SCOPE_END 274
#define TK_SL2VL_TABLES_START 275
#define TK_SL2VL_TABLES_END 276
#define TK_SL2VL_SCOPE_START 277
#define TK_SL2VL_SCOPE_END 278
#define TK_QOS_LEVELS_START 279
#define TK_QOS_LEVELS_END 280
#define TK_QOS_LEVEL_START 281
#define TK_QOS_LEVEL_END 282
#define TK_QOS_MATCH_RULES_START 283
#define TK_QOS_MATCH_RULES_END 284
#define TK_QOS_MATCH_RULE_START 285
#define TK_QOS_MATCH_RULE_END 286
#define TK_NAME 287
#define TK_USE 288
#define TK_PORT_GUID 289
#define TK_PORT_NAME 290
#define TK_PARTITION 291
#define TK_NODE_TYPE 292
#define TK_GROUP 293
#define TK_ACROSS 294
#define TK_VLARB_HIGH 295
#define TK_VLARB_LOW 296
#define TK_VLARB_HIGH_LIMIT 297
#define TK_TO 298
#define TK_FROM 299
#define TK_ACROSS_TO 300
#define TK_ACROSS_FROM 301
#define TK_SL2VL_TABLE 302
#define TK_SL 303
#define TK_MTU_LIMIT 304
#define TK_RATE_LIMIT 305
#define TK_PACKET_LIFE 306
#define TK_PATH_BITS 307
#define TK_QOS_CLASS 308
#define TK_SOURCE 309
#define TK_DESTINATION 310
#define TK_SERVICE_ID 311
#define TK_QOS_LEVEL_NAME 312
#define TK_PKEY 313
#define TK_NODE_TYPE_ROUTER 314
#define TK_NODE_TYPE_CA 315
#define TK_NODE_TYPE_SWITCH 316
#define TK_NODE_TYPE_SELF 317
#define TK_NODE_TYPE_ALL 318
#define TK_ULP_DEFAULT 319
#define TK_ULP_ANY_SERVICE_ID 320
#define TK_ULP_ANY_PKEY 321
#define TK_ULP_ANY_TARGET_PORT_GUID 322
#define TK_ULP_ANY_SOURCE_PORT_GUID 323
#define TK_ULP_ANY_SOURCE_TARGET_PORT_GUID 324
#define TK_ULP_SDP_DEFAULT 325
#define TK_ULP_SDP_PORT 326
#define TK_ULP_RDS_DEFAULT 327
#define TK_ULP_RDS_PORT 328
#define TK_ULP_ISER_DEFAULT 329
#define TK_ULP_ISER_PORT 330
#define TK_ULP_SRP_GUID 331
#define TK_ULP_IPOIB_DEFAULT 332
#define TK_ULP_IPOIB_PKEY 333
#define YYERRCODE 256
typedef int YYINT;
static const YYINT yylhs[] = {                           -1,
    0,    1,    1,    2,    2,    2,    2,    2,    3,    8,
    8,    4,   10,   10,   11,   12,   14,   13,   13,   15,
   15,   15,   15,   15,   15,   15,    5,   23,   23,   23,
   24,   26,   26,   27,   28,   30,   29,   29,   31,   31,
   31,   31,   31,   25,   37,   37,   38,   39,   41,   40,
   40,   42,   42,   42,   42,   42,   42,   42,    6,   50,
   50,   51,   52,   54,   53,   53,   55,   55,   55,   55,
   55,   55,   55,   55,    7,   64,   64,   65,   66,   68,
   67,   67,   69,   69,   69,   69,   69,   69,   69,    9,
   81,    9,   83,    9,   85,    9,   87,    9,   89,    9,
   91,    9,   93,    9,   95,    9,   97,    9,   99,    9,
  101,    9,  103,    9,  105,    9,  107,    9,   78,   82,
   84,   86,   88,   90,   92,   94,   96,   98,  100,  102,
  104,  106,   80,   16,  108,   17,  110,   19,  111,   18,
  113,   20,  114,   21,  115,   22,  116,  117,  117,  118,
  118,  118,  118,  118,  119,  120,  121,  122,  123,   32,
  124,   33,  125,   36,  126,   34,  127,   35,  129,   43,
  130,   44,  131,   45,  132,   46,  133,   47,  134,   48,
  136,  135,  135,  138,  137,  137,  140,  139,  141,   49,
  142,   56,  144,   57,  145,   58,  146,   59,  147,   60,
  148,   61,  149,   62,  150,   63,  151,   70,  152,   71,
  153,   73,  154,   74,  155,   72,  156,   75,  157,   76,
  158,  109,  159,  159,  160,  112,  112,   77,  143,  143,
  161,  128,  128,  162,  163,   79,  164,  164,  164,  164,
  165,  166,  167,
};
static const YYINT yylen[] = {                            2,
    1,    0,    2,    1,    1,    1,    1,    1,    3,    1,
    2,    3,    1,    2,    3,    1,    1,    0,    2,    1,
    1,    1,    1,    1,    1,    1,    3,    0,    2,    2,
    3,    0,    2,    3,    1,    1,    0,    2,    1,    1,
    1,    1,    1,    3,    0,    2,    3,    1,    1,    0,
    2,    1,    1,    1,    1,    1,    1,    1,    3,    0,
    2,    3,    1,    1,    0,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    3,    0,    2,    3,    1,    1,
    0,    2,    1,    1,    1,    1,    1,    1,    1,    2,
    0,    5,    0,    5,    0,    5,    0,    5,    0,    5,
    0,    3,    0,    5,    0,    3,    0,    5,    0,    3,
    0,    5,    0,    5,    0,    3,    0,    5,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    1,    3,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    1,    1,    2,    1,    1,    3,    1,    1,    3,
    1,    3,    5,    1,    1,    1,    1,    3,    5,    3,
    1,    1,    1,
};
static const YYINT yydefred[] = {                         2,
    0,    0,    0,    0,   28,   60,   76,    3,    4,    5,
    6,    7,    8,    0,  119,  120,  121,  122,  123,  124,
  125,  126,  127,  128,  129,  130,  131,  132,    0,   10,
    0,    0,    0,    0,    0,  101,    0,  105,    0,  109,
    0,    0,  115,    0,   16,    0,   13,   18,    0,    0,
    0,  231,   90,  228,    9,   11,    0,    0,    0,  237,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   12,   14,    0,   27,   32,   45,
   29,   30,   59,   63,   61,   65,   75,   79,   77,   81,
   91,    0,    0,   93,   95,   97,   99,  133,  102,  103,
  106,  107,  110,  111,  113,  116,  117,   17,  135,  137,
  141,  139,  145,  147,  143,   15,   19,   20,   21,   22,
   23,   24,   25,   26,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  240,    0,  243,  238,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  225,
  134,    0,  223,  136,  226,    0,  140,  142,    0,  157,
  155,  156,  159,  158,    0,  148,  150,  151,  152,  153,
  154,   31,   35,   33,   37,   44,   48,   46,   50,   64,
  193,  195,  197,  199,  201,  203,  205,  207,   62,   66,
   67,   68,   69,   70,   71,   72,   73,   74,    0,    0,
    0,    0,    0,    0,    0,    0,   80,  209,  211,  213,
  215,  219,  217,  221,   78,   82,   83,   84,   85,   86,
   87,   88,   89,    0,    0,    0,    0,    0,    0,    0,
   92,    0,   94,   96,   98,  100,  104,  108,  112,  114,
  118,  224,    0,    0,    0,    0,  192,  194,  196,  198,
  200,  202,  204,  206,  208,  210,    0,    0,  216,  218,
  220,  239,  227,  149,   36,  161,  163,  167,  169,  165,
   34,   38,   39,   40,   41,   42,   43,    0,    0,    0,
    0,    0,   49,  171,  173,  181,  179,  177,  175,  191,
   47,   51,   52,   53,   54,   55,   56,   57,   58,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  164,  234,
    0,    0,    0,    0,    0,    0,    0,  184,  188,  178,
  182,  183,  187,  189,  180,  185,  186,    0,  229,    0,
    0,    0,    0,  235,  232,  230,    0,  233,
};
static const YYINT yydgoto[] = {                          1,
    2,    8,    9,   10,   11,   12,   13,   29,   30,   46,
   47,   48,   77,  116,  117,  118,  119,  120,  121,  122,
  123,  124,   49,   81,   82,  132,  174,  175,  245,  271,
  272,  273,  274,  275,  276,  277,  133,  178,  179,  246,
  291,  292,  293,  294,  295,  296,  297,  298,  299,   50,
   85,   86,  134,  189,  190,  191,  192,  193,  194,  195,
  196,  197,  198,   51,   89,   90,  135,  215,  216,  217,
  218,  219,  220,  221,  222,  223,   98,   31,   58,   99,
  136,   32,  141,   33,  142,   34,  143,   35,  144,   36,
   66,   37,  145,   38,   68,   39,  146,   40,   70,   41,
  147,   42,  148,   43,   73,   44,  149,  125,  155,  126,
  127,  156,  128,  129,  130,  131,  165,  166,  167,  168,
  169,  170,  171,  278,  279,  280,  281,  311,  282,  300,
  301,  302,  303,  304,  320,  305,  325,  321,  322,  326,
  327,  306,  328,  199,  200,  201,  202,  203,  204,  205,
  206,  224,  225,  226,  227,  228,  229,  230,  152,  153,
   54,  312,  335,   59,   60,   61,  140,
};
static const YYINT yysindex[] = {                         0,
    0,  -73,  147, -256,    0,    0,    0,    0,    0,    0,
    0,    0,    0, -238,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -260,    0,
 -230, -230, -230, -230, -230,    0, -230,    0, -230,    0,
 -230, -230,    0, -230,    0, -249,    0,    0, -245, -226,
 -227,    0,    0,    0,    0,    0,    0, -224, -170,    0,
 -165, -164, -145, -144, -143, -238, -140, -238, -133, -238,
 -131, -130, -238, -129,    0,    0, -239,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -230, -163,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -128, -128, -128, -230, -230, -128,
 -233, -186, -185, -266, -188, -238,    0, -127,    0,    0,
 -238, -238, -238, -238, -238, -238, -238, -238, -238,    0,
    0, -128,    0,    0,    0, -125,    0,    0, -125,    0,
    0,    0,    0,    0, -122,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -128, -128,
 -238, -238, -238, -238, -230, -230,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, -128, -230, -128, -128, -128, -230, -230,
    0, -163,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -128, -233, -251,   69,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, -125, -125,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -128, -128, -238,
 -121, -121,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -128,
 -128, -128, -128, -237, -229, -238, -125, -125,    0,    0,
 -120, -126, -120, -125, -125, -125, -125,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -118,    0, -121,
 -114, -238, -115,    0,    0,    0, -114,    0,
};
static const YYINT yyrindex[] = {                         0,
    0,  139,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  -74,    0, -141,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   -4,    0,    0,    0, -212,    0,    0,  -47,    0,
    0,    0,    0,    0,   42,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   27,   33,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  144,  154,    0,    0,
  162,    0,  168,   79,   96,  106,  123,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  133,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,
};
static const YYINT yygindex[] = {                         0,
    0,    0,    0,    0,    0,    0,    0,    0,  127,    0,
  129,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   -1,    0,  -32,  -37,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -111,    0,
    0, -124,    0,    0,    0,    0,    0,  -64,    0,    0,
    0,    0,    0,    0,    0,    0,    0, -101,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   30,
 -298, -147, -150,    0,   97,   99,  -44,
};
#define YYTABLESIZE 480
static const YYINT yytable[] = {                         62,
   63,   64,   65,   55,   67,  159,   69,  329,   71,   72,
   45,   74,   53,  151,  154,  180,   75,   45,   52,   57,
  181,  182,  265,  318,   78,   79,   57,   57,  108,   80,
  101,  323,  103,  336,   91,  106,  183,  184,  185,  186,
  187,  266,  267,  268,  269,  270,  188,  109,  110,  111,
  112,  113,  114,   83,   84,  138,   87,   88,   14,   15,
   16,   17,   18,   19,   20,   21,   22,   23,   24,   25,
   26,   27,   28,  115,  138,  138,  138,  138,  138,  138,
  160,  161,  162,  163,  164,  172,  173,  247,  248,   92,
  176,  177,   93,  139,   94,  157,  158,  207,  231,  208,
  138,  257,  258,  233,  234,  235,  236,  237,  238,  239,
  240,  241,  255,   95,   96,   97,  259,  236,  100,  209,
  210,  211,  212,  213,  214,  102,  236,  104,  105,  107,
  232,  263,  331,  150,  243,  310,  236,  244,    1,  330,
  236,  332,  334,  337,  236,  236,  236,  236,  236,  236,
  236,  236,  236,  307,  308,   56,  236,  236,  236,  236,
  236,  236,  236,  236,  236,  236,  236,  236,  236,  236,
  236,  236,  253,  254,   76,  314,  315,  316,  317,  264,
  313,  242,  333,  242,  241,  241,  338,  262,  137,    3,
  138,    4,  256,  241,    0,    5,  260,  261,    0,  249,
  250,  251,  252,  241,    0,    6,    0,  241,    0,    7,
    0,  241,  241,  241,  241,  241,  241,  241,  241,  241,
  144,    0,    0,  241,  241,  241,  241,  241,  241,  241,
  241,  241,  241,  241,  241,  241,  241,  241,  241,  144,
  144,  144,  144,  144,  144,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  222,    0,    0,    0,    0,
    0,    0,    0,  222,    0,  144,    0,    0,    0,  222,
    0,  319,  324,  222,    0,    0,    0,  222,  309,    0,
    0,  222,  222,  222,  222,  222,  222,  222,  222,  222,
  222,  222,  222,  222,  222,  222,  222,  222,  222,  222,
  222,  222,  222,  222,  222,  222,  222,  222,  222,  146,
    0,    0,  212,    0,  212,    0,    0,    0,  214,    0,
  214,    0,    0,    0,    0,    0,    0,    0,  146,  146,
  146,  146,  146,  146,  212,  212,  212,  212,  212,  212,
  214,  214,  214,  214,  214,  214,  283,    0,    0,    0,
    0,    0,    0,    0,  146,    0,  170,    0,    0,    0,
    0,  284,  285,    0,    0,    0,  286,  287,  288,  289,
  290,  170,  170,  172,    0,    0,  170,  170,  170,  170,
  170,    0,    0,  174,    0,    0,    0,    0,  172,  172,
    0,    0,    0,  172,  172,  172,  172,  172,  174,  174,
  176,    0,    0,  174,  174,  174,  174,  174,    0,    0,
  190,    0,    0,    0,    0,  176,  176,  160,    0,    0,
  176,  176,  176,  176,  176,  190,  190,  162,    0,    0,
  190,  190,  190,  190,  190,  166,  160,  160,  160,  160,
  160,  168,    0,    0,    0,    0,  162,  162,  162,  162,
  162,    0,    0,    0,  166,  166,  166,  166,  166,    0,
  168,  168,  168,  168,  168,   14,   15,   16,   17,   18,
   19,   20,   21,   22,   23,   24,   25,   26,   27,   28,
};
static const YYINT yycheck[] = {                         32,
   33,   34,   35,  264,   37,  130,   39,  306,   41,   42,
  267,   44,   14,  125,  126,  282,  266,  267,  257,  257,
  287,  288,  274,  261,  270,  271,  257,  257,  268,  275,
   68,  261,   70,  332,  259,   73,  303,  304,  305,  306,
  307,  293,  294,  295,  296,  297,  313,  287,  288,  289,
  290,  291,  292,  280,  281,  268,  284,  285,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
  331,  332,  333,  313,  287,  288,  289,  290,  291,  292,
  314,  315,  316,  317,  318,  272,  273,  199,  200,  260,
  276,  277,  258,  257,  259,  128,  129,  286,  136,  288,
  313,  226,  227,  141,  142,  143,  144,  145,  146,  147,
  148,  149,  224,  259,  259,  259,  228,  259,  259,  308,
  309,  310,  311,  312,  313,  259,  268,  259,  259,  259,
  258,  243,  259,  262,  260,  257,  278,  260,    0,  260,
  282,  260,  257,  259,  286,  287,  288,  289,  290,  291,
  292,  293,  294,  278,  279,   29,  298,  299,  300,  301,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
  312,  313,  205,  206,   46,  300,  301,  302,  303,  244,
  282,  152,  330,  258,  259,  260,  337,  232,   92,  263,
   92,  265,  225,  268,   -1,  269,  229,  230,   -1,  201,
  202,  203,  204,  278,   -1,  279,   -1,  282,   -1,  283,
   -1,  286,  287,  288,  289,  290,  291,  292,  293,  294,
  268,   -1,   -1,  298,  299,  300,  301,  302,  303,  304,
  305,  306,  307,  308,  309,  310,  311,  312,  313,  287,
  288,  289,  290,  291,  292,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  260,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  268,   -1,  313,   -1,   -1,   -1,  274,
   -1,  304,  305,  278,   -1,   -1,   -1,  282,  280,   -1,
   -1,  286,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296,  297,  298,  299,  300,  301,  302,  303,  304,
  305,  306,  307,  308,  309,  310,  311,  312,  313,  268,
   -1,   -1,  286,   -1,  288,   -1,   -1,   -1,  286,   -1,
  288,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  287,  288,
  289,  290,  291,  292,  308,  309,  310,  311,  312,  313,
  308,  309,  310,  311,  312,  313,  278,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  313,   -1,  278,   -1,   -1,   -1,
   -1,  293,  294,   -1,   -1,   -1,  298,  299,  300,  301,
  302,  293,  294,  278,   -1,   -1,  298,  299,  300,  301,
  302,   -1,   -1,  278,   -1,   -1,   -1,   -1,  293,  294,
   -1,   -1,   -1,  298,  299,  300,  301,  302,  293,  294,
  278,   -1,   -1,  298,  299,  300,  301,  302,   -1,   -1,
  278,   -1,   -1,   -1,   -1,  293,  294,  274,   -1,   -1,
  298,  299,  300,  301,  302,  293,  294,  274,   -1,   -1,
  298,  299,  300,  301,  302,  274,  293,  294,  295,  296,
  297,  274,   -1,   -1,   -1,   -1,  293,  294,  295,  296,
  297,   -1,   -1,   -1,  293,  294,  295,  296,  297,   -1,
  293,  294,  295,  296,  297,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,  331,  332,  333,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 333
#define YYUNDFTOKEN 503
#define YYTRANSLATE(a) ((a) > YYMAXTOKEN ? YYUNDFTOKEN : (a))
#if YYDEBUG
static const char *const yyname[] = {

"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"TK_NUMBER","TK_DASH","TK_DOTDOT",
"TK_COMMA","TK_ASTERISK","TK_TEXT","TK_QOS_ULPS_START","TK_QOS_ULPS_END",
"TK_PORT_GROUPS_START","TK_PORT_GROUPS_END","TK_PORT_GROUP_START",
"TK_PORT_GROUP_END","TK_QOS_SETUP_START","TK_QOS_SETUP_END",
"TK_VLARB_TABLES_START","TK_VLARB_TABLES_END","TK_VLARB_SCOPE_START",
"TK_VLARB_SCOPE_END","TK_SL2VL_TABLES_START","TK_SL2VL_TABLES_END",
"TK_SL2VL_SCOPE_START","TK_SL2VL_SCOPE_END","TK_QOS_LEVELS_START",
"TK_QOS_LEVELS_END","TK_QOS_LEVEL_START","TK_QOS_LEVEL_END",
"TK_QOS_MATCH_RULES_START","TK_QOS_MATCH_RULES_END","TK_QOS_MATCH_RULE_START",
"TK_QOS_MATCH_RULE_END","TK_NAME","TK_USE","TK_PORT_GUID","TK_PORT_NAME",
"TK_PARTITION","TK_NODE_TYPE","TK_GROUP","TK_ACROSS","TK_VLARB_HIGH",
"TK_VLARB_LOW","TK_VLARB_HIGH_LIMIT","TK_TO","TK_FROM","TK_ACROSS_TO",
"TK_ACROSS_FROM","TK_SL2VL_TABLE","TK_SL","TK_MTU_LIMIT","TK_RATE_LIMIT",
"TK_PACKET_LIFE","TK_PATH_BITS","TK_QOS_CLASS","TK_SOURCE","TK_DESTINATION",
"TK_SERVICE_ID","TK_QOS_LEVEL_NAME","TK_PKEY","TK_NODE_TYPE_ROUTER",
"TK_NODE_TYPE_CA","TK_NODE_TYPE_SWITCH","TK_NODE_TYPE_SELF","TK_NODE_TYPE_ALL",
"TK_ULP_DEFAULT","TK_ULP_ANY_SERVICE_ID","TK_ULP_ANY_PKEY",
"TK_ULP_ANY_TARGET_PORT_GUID","TK_ULP_ANY_SOURCE_PORT_GUID",
"TK_ULP_ANY_SOURCE_TARGET_PORT_GUID","TK_ULP_SDP_DEFAULT","TK_ULP_SDP_PORT",
"TK_ULP_RDS_DEFAULT","TK_ULP_RDS_PORT","TK_ULP_ISER_DEFAULT","TK_ULP_ISER_PORT",
"TK_ULP_SRP_GUID","TK_ULP_IPOIB_DEFAULT","TK_ULP_IPOIB_PKEY",0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"illegal-symbol",
};
static const char *const yyrule[] = {
"$accept : head",
"head : qos_policy_entries",
"qos_policy_entries :",
"qos_policy_entries : qos_policy_entries qos_policy_entry",
"qos_policy_entry : qos_ulps_section",
"qos_policy_entry : port_groups_section",
"qos_policy_entry : qos_setup_section",
"qos_policy_entry : qos_levels_section",
"qos_policy_entry : qos_match_rules_section",
"qos_ulps_section : TK_QOS_ULPS_START qos_ulps TK_QOS_ULPS_END",
"qos_ulps : qos_ulp",
"qos_ulps : qos_ulps qos_ulp",
"port_groups_section : TK_PORT_GROUPS_START port_groups TK_PORT_GROUPS_END",
"port_groups : port_group",
"port_groups : port_groups port_group",
"port_group : port_group_start port_group_entries port_group_end",
"port_group_start : TK_PORT_GROUP_START",
"port_group_end : TK_PORT_GROUP_END",
"port_group_entries :",
"port_group_entries : port_group_entries port_group_entry",
"port_group_entry : port_group_name",
"port_group_entry : port_group_use",
"port_group_entry : port_group_port_guid",
"port_group_entry : port_group_port_name",
"port_group_entry : port_group_pkey",
"port_group_entry : port_group_partition",
"port_group_entry : port_group_node_type",
"qos_setup_section : TK_QOS_SETUP_START qos_setup_items TK_QOS_SETUP_END",
"qos_setup_items :",
"qos_setup_items : qos_setup_items vlarb_tables",
"qos_setup_items : qos_setup_items sl2vl_tables",
"vlarb_tables : TK_VLARB_TABLES_START vlarb_scope_items TK_VLARB_TABLES_END",
"vlarb_scope_items :",
"vlarb_scope_items : vlarb_scope_items vlarb_scope",
"vlarb_scope : vlarb_scope_start vlarb_scope_entries vlarb_scope_end",
"vlarb_scope_start : TK_VLARB_SCOPE_START",
"vlarb_scope_end : TK_VLARB_SCOPE_END",
"vlarb_scope_entries :",
"vlarb_scope_entries : vlarb_scope_entries vlarb_scope_entry",
"vlarb_scope_entry : vlarb_scope_group",
"vlarb_scope_entry : vlarb_scope_across",
"vlarb_scope_entry : vlarb_scope_vlarb_high",
"vlarb_scope_entry : vlarb_scope_vlarb_low",
"vlarb_scope_entry : vlarb_scope_vlarb_high_limit",
"sl2vl_tables : TK_SL2VL_TABLES_START sl2vl_scope_items TK_SL2VL_TABLES_END",
"sl2vl_scope_items :",
"sl2vl_scope_items : sl2vl_scope_items sl2vl_scope",
"sl2vl_scope : sl2vl_scope_start sl2vl_scope_entries sl2vl_scope_end",
"sl2vl_scope_start : TK_SL2VL_SCOPE_START",
"sl2vl_scope_end : TK_SL2VL_SCOPE_END",
"sl2vl_scope_entries :",
"sl2vl_scope_entries : sl2vl_scope_entries sl2vl_scope_entry",
"sl2vl_scope_entry : sl2vl_scope_group",
"sl2vl_scope_entry : sl2vl_scope_across",
"sl2vl_scope_entry : sl2vl_scope_across_from",
"sl2vl_scope_entry : sl2vl_scope_across_to",
"sl2vl_scope_entry : sl2vl_scope_from",
"sl2vl_scope_entry : sl2vl_scope_to",
"sl2vl_scope_entry : sl2vl_scope_sl2vl_table",
"qos_levels_section : TK_QOS_LEVELS_START qos_levels TK_QOS_LEVELS_END",
"qos_levels :",
"qos_levels : qos_levels qos_level",
"qos_level : qos_level_start qos_level_entries qos_level_end",
"qos_level_start : TK_QOS_LEVEL_START",
"qos_level_end : TK_QOS_LEVEL_END",
"qos_level_entries :",
"qos_level_entries : qos_level_entries qos_level_entry",
"qos_level_entry : qos_level_name",
"qos_level_entry : qos_level_use",
"qos_level_entry : qos_level_sl",
"qos_level_entry : qos_level_mtu_limit",
"qos_level_entry : qos_level_rate_limit",
"qos_level_entry : qos_level_packet_life",
"qos_level_entry : qos_level_path_bits",
"qos_level_entry : qos_level_pkey",
"qos_match_rules_section : TK_QOS_MATCH_RULES_START qos_match_rules TK_QOS_MATCH_RULES_END",
"qos_match_rules :",
"qos_match_rules : qos_match_rules qos_match_rule",
"qos_match_rule : qos_match_rule_start qos_match_rule_entries qos_match_rule_end",
"qos_match_rule_start : TK_QOS_MATCH_RULE_START",
"qos_match_rule_end : TK_QOS_MATCH_RULE_END",
"qos_match_rule_entries :",
"qos_match_rule_entries : qos_match_rule_entries qos_match_rule_entry",
"qos_match_rule_entry : qos_match_rule_use",
"qos_match_rule_entry : qos_match_rule_qos_class",
"qos_match_rule_entry : qos_match_rule_qos_level_name",
"qos_match_rule_entry : qos_match_rule_source",
"qos_match_rule_entry : qos_match_rule_destination",
"qos_match_rule_entry : qos_match_rule_service_id",
"qos_match_rule_entry : qos_match_rule_pkey",
"qos_ulp : TK_ULP_DEFAULT single_number",
"$$1 :",
"qos_ulp : qos_ulp_type_any_service list_of_ranges TK_DOTDOT $$1 qos_ulp_sl",
"$$2 :",
"qos_ulp : qos_ulp_type_any_pkey list_of_ranges TK_DOTDOT $$2 qos_ulp_sl",
"$$3 :",
"qos_ulp : qos_ulp_type_any_target_port_guid list_of_ranges TK_DOTDOT $$3 qos_ulp_sl",
"$$4 :",
"qos_ulp : qos_ulp_type_any_source_port_guid list_of_ranges TK_DOTDOT $$4 qos_ulp_sl",
"$$5 :",
"qos_ulp : qos_ulp_type_any_source_target_port_guid list_of_ranges TK_DOTDOT $$5 qos_ulp_sl",
"$$6 :",
"qos_ulp : qos_ulp_type_sdp_default $$6 qos_ulp_sl",
"$$7 :",
"qos_ulp : qos_ulp_type_sdp_port list_of_ranges TK_DOTDOT $$7 qos_ulp_sl",
"$$8 :",
"qos_ulp : qos_ulp_type_rds_default $$8 qos_ulp_sl",
"$$9 :",
"qos_ulp : qos_ulp_type_rds_port list_of_ranges TK_DOTDOT $$9 qos_ulp_sl",
"$$10 :",
"qos_ulp : qos_ulp_type_iser_default $$10 qos_ulp_sl",
"$$11 :",
"qos_ulp : qos_ulp_type_iser_port list_of_ranges TK_DOTDOT $$11 qos_ulp_sl",
"$$12 :",
"qos_ulp : qos_ulp_type_srp_guid list_of_ranges TK_DOTDOT $$12 qos_ulp_sl",
"$$13 :",
"qos_ulp : qos_ulp_type_ipoib_default $$13 qos_ulp_sl",
"$$14 :",
"qos_ulp : qos_ulp_type_ipoib_pkey list_of_ranges TK_DOTDOT $$14 qos_ulp_sl",
"qos_ulp_type_any_service : TK_ULP_ANY_SERVICE_ID",
"qos_ulp_type_any_pkey : TK_ULP_ANY_PKEY",
"qos_ulp_type_any_target_port_guid : TK_ULP_ANY_TARGET_PORT_GUID",
"qos_ulp_type_any_source_port_guid : TK_ULP_ANY_SOURCE_PORT_GUID",
"qos_ulp_type_any_source_target_port_guid : TK_ULP_ANY_SOURCE_TARGET_PORT_GUID",
"qos_ulp_type_sdp_default : TK_ULP_SDP_DEFAULT",
"qos_ulp_type_sdp_port : TK_ULP_SDP_PORT",
"qos_ulp_type_rds_default : TK_ULP_RDS_DEFAULT",
"qos_ulp_type_rds_port : TK_ULP_RDS_PORT",
"qos_ulp_type_iser_default : TK_ULP_ISER_DEFAULT",
"qos_ulp_type_iser_port : TK_ULP_ISER_PORT",
"qos_ulp_type_srp_guid : TK_ULP_SRP_GUID",
"qos_ulp_type_ipoib_default : TK_ULP_IPOIB_DEFAULT",
"qos_ulp_type_ipoib_pkey : TK_ULP_IPOIB_PKEY",
"qos_ulp_sl : single_number",
"port_group_name : port_group_name_start single_string",
"port_group_name_start : TK_NAME",
"port_group_use : port_group_use_start single_string",
"port_group_use_start : TK_USE",
"port_group_port_name : port_group_port_name_start string_list",
"port_group_port_name_start : TK_PORT_NAME",
"port_group_port_guid : port_group_port_guid_start list_of_ranges",
"port_group_port_guid_start : TK_PORT_GUID",
"port_group_pkey : port_group_pkey_start list_of_ranges",
"port_group_pkey_start : TK_PKEY",
"port_group_partition : port_group_partition_start string_list",
"port_group_partition_start : TK_PARTITION",
"port_group_node_type : port_group_node_type_start port_group_node_type_list",
"port_group_node_type_start : TK_NODE_TYPE",
"port_group_node_type_list : node_type_item",
"port_group_node_type_list : port_group_node_type_list TK_COMMA node_type_item",
"node_type_item : node_type_ca",
"node_type_item : node_type_switch",
"node_type_item : node_type_router",
"node_type_item : node_type_all",
"node_type_item : node_type_self",
"node_type_ca : TK_NODE_TYPE_CA",
"node_type_switch : TK_NODE_TYPE_SWITCH",
"node_type_router : TK_NODE_TYPE_ROUTER",
"node_type_all : TK_NODE_TYPE_ALL",
"node_type_self : TK_NODE_TYPE_SELF",
"vlarb_scope_group : vlarb_scope_group_start string_list",
"vlarb_scope_group_start : TK_GROUP",
"vlarb_scope_across : vlarb_scope_across_start string_list",
"vlarb_scope_across_start : TK_ACROSS",
"vlarb_scope_vlarb_high_limit : vlarb_scope_vlarb_high_limit_start single_number",
"vlarb_scope_vlarb_high_limit_start : TK_VLARB_HIGH_LIMIT",
"vlarb_scope_vlarb_high : vlarb_scope_vlarb_high_start num_list_with_dotdot",
"vlarb_scope_vlarb_high_start : TK_VLARB_HIGH",
"vlarb_scope_vlarb_low : vlarb_scope_vlarb_low_start num_list_with_dotdot",
"vlarb_scope_vlarb_low_start : TK_VLARB_LOW",
"sl2vl_scope_group : sl2vl_scope_group_start string_list",
"sl2vl_scope_group_start : TK_GROUP",
"sl2vl_scope_across : sl2vl_scope_across_start string_list",
"sl2vl_scope_across_start : TK_ACROSS",
"sl2vl_scope_across_from : sl2vl_scope_across_from_start string_list",
"sl2vl_scope_across_from_start : TK_ACROSS_FROM",
"sl2vl_scope_across_to : sl2vl_scope_across_to_start string_list",
"sl2vl_scope_across_to_start : TK_ACROSS_TO",
"sl2vl_scope_from : sl2vl_scope_from_start sl2vl_scope_from_list_or_asterisk",
"sl2vl_scope_from_start : TK_FROM",
"sl2vl_scope_to : sl2vl_scope_to_start sl2vl_scope_to_list_or_asterisk",
"sl2vl_scope_to_start : TK_TO",
"sl2vl_scope_from_list_or_asterisk : sl2vl_scope_from_asterisk",
"sl2vl_scope_from_list_or_asterisk : sl2vl_scope_from_list_of_ranges",
"sl2vl_scope_from_asterisk : TK_ASTERISK",
"sl2vl_scope_to_list_or_asterisk : sl2vl_scope_to_asterisk",
"sl2vl_scope_to_list_or_asterisk : sl2vl_scope_to_list_of_ranges",
"sl2vl_scope_to_asterisk : TK_ASTERISK",
"sl2vl_scope_from_list_of_ranges : list_of_ranges",
"sl2vl_scope_to_list_of_ranges : list_of_ranges",
"sl2vl_scope_sl2vl_table : sl2vl_scope_sl2vl_table_start num_list",
"sl2vl_scope_sl2vl_table_start : TK_SL2VL_TABLE",
"qos_level_name : qos_level_name_start single_string",
"qos_level_name_start : TK_NAME",
"qos_level_use : qos_level_use_start single_string",
"qos_level_use_start : TK_USE",
"qos_level_sl : qos_level_sl_start single_number",
"qos_level_sl_start : TK_SL",
"qos_level_mtu_limit : qos_level_mtu_limit_start single_number",
"qos_level_mtu_limit_start : TK_MTU_LIMIT",
"qos_level_rate_limit : qos_level_rate_limit_start single_number",
"qos_level_rate_limit_start : TK_RATE_LIMIT",
"qos_level_packet_life : qos_level_packet_life_start single_number",
"qos_level_packet_life_start : TK_PACKET_LIFE",
"qos_level_path_bits : qos_level_path_bits_start list_of_ranges",
"qos_level_path_bits_start : TK_PATH_BITS",
"qos_level_pkey : qos_level_pkey_start list_of_ranges",
"qos_level_pkey_start : TK_PKEY",
"qos_match_rule_use : qos_match_rule_use_start single_string",
"qos_match_rule_use_start : TK_USE",
"qos_match_rule_qos_class : qos_match_rule_qos_class_start list_of_ranges",
"qos_match_rule_qos_class_start : TK_QOS_CLASS",
"qos_match_rule_source : qos_match_rule_source_start string_list",
"qos_match_rule_source_start : TK_SOURCE",
"qos_match_rule_destination : qos_match_rule_destination_start string_list",
"qos_match_rule_destination_start : TK_DESTINATION",
"qos_match_rule_qos_level_name : qos_match_rule_qos_level_name_start single_string",
"qos_match_rule_qos_level_name_start : TK_QOS_LEVEL_NAME",
"qos_match_rule_service_id : qos_match_rule_service_id_start list_of_ranges",
"qos_match_rule_service_id_start : TK_SERVICE_ID",
"qos_match_rule_pkey : qos_match_rule_pkey_start list_of_ranges",
"qos_match_rule_pkey_start : TK_PKEY",
"single_string : single_string_elems",
"single_string_elems : single_string_element",
"single_string_elems : single_string_elems single_string_element",
"single_string_element : TK_TEXT",
"string_list : single_string",
"string_list : string_list TK_COMMA single_string",
"single_number : number",
"num_list : number",
"num_list : num_list TK_COMMA number",
"number : TK_NUMBER",
"num_list_with_dotdot : number_from_pair_1 TK_DOTDOT number_from_pair_2",
"num_list_with_dotdot : num_list_with_dotdot TK_COMMA number_from_pair_1 TK_DOTDOT number_from_pair_2",
"number_from_pair_1 : TK_NUMBER",
"number_from_pair_2 : TK_NUMBER",
"list_of_ranges : num_list_with_dash",
"num_list_with_dash : single_number_from_range",
"num_list_with_dash : number_from_range_1 TK_DASH number_from_range_2",
"num_list_with_dash : num_list_with_dash TK_COMMA number_from_range_1 TK_DASH number_from_range_2",
"num_list_with_dash : num_list_with_dash TK_COMMA single_number_from_range",
"single_number_from_range : TK_NUMBER",
"number_from_range_1 : TK_NUMBER",
"number_from_range_2 : TK_NUMBER",

};
#endif

#if YYDEBUG
int      yydebug;
#endif

int      yyerrflag;
int      yychar;
YYSTYPE  yyval;
YYSTYPE  yylval;
int      yynerrs;

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#define YYINITSTACKSIZE 200

typedef struct {
    unsigned stacksize;
    YYINT    *s_base;
    YYINT    *s_mark;
    YYINT    *s_last;
    YYSTYPE  *l_base;
    YYSTYPE  *l_mark;
} YYSTACKDATA;
/* variables for the parser stack */
static YYSTACKDATA yystack;
#line 2379 "osm_qos_parser_y.y"

/***************************************************
 ***************************************************/

int osm_qos_parse_policy_file(IN osm_subn_t * p_subn)
{
    int res = 0;
    static boolean_t first_time = TRUE;
    p_qos_parser_osm_log = &p_subn->p_osm->log;

    OSM_LOG_ENTER(p_qos_parser_osm_log);

    osm_qos_policy_destroy(p_subn->p_qos_policy);
    p_subn->p_qos_policy = NULL;

    if (!p_subn->opt.qos_policy_file) {
        OSM_LOG(p_qos_parser_osm_log, OSM_LOG_ERROR, "ERR AC06: "
                "QoS policy file name is empty\n");
        res = 1;
        goto Exit;
    }

    yyin = fopen (p_subn->opt.qos_policy_file, "r");
    if (!yyin)
    {
        if (strcmp(p_subn->opt.qos_policy_file,OSM_DEFAULT_QOS_POLICY_FILE)) {
            OSM_LOG(p_qos_parser_osm_log, OSM_LOG_ERROR, "ERR AC01: "
                    "Failed opening QoS policy file %s - %s\n",
                    p_subn->opt.qos_policy_file, strerror(errno));
            res = 1;
        }
        else
            OSM_LOG(p_qos_parser_osm_log, OSM_LOG_VERBOSE,
                    "QoS policy file not found (%s)\n",
                    p_subn->opt.qos_policy_file);

        goto Exit;
    }

    if (first_time)
    {
        first_time = FALSE;
        __setup_simple_qos_levels();
        __setup_ulp_match_rules();
        OSM_LOG(p_qos_parser_osm_log, OSM_LOG_INFO,
		"Loading QoS policy file (%s)\n",
                p_subn->opt.qos_policy_file);
    }
    else
        /*
         * ULP match rules list was emptied at the end of
         * previous parsing iteration.
         * What's left is to clear simple QoS levels.
         */
        __clear_simple_qos_levels();

    column_num = 1;
    line_num = 1;

    p_subn->p_qos_policy = osm_qos_policy_create(p_subn);

    __parser_tmp_struct_init();
    p_qos_policy = p_subn->p_qos_policy;

    res = yyparse();

    __parser_tmp_struct_destroy();

    if (res != 0)
    {
        OSM_LOG(p_qos_parser_osm_log, OSM_LOG_ERROR, "ERR AC03: "
                "Failed parsing QoS policy file (%s)\n",
                p_subn->opt.qos_policy_file);
        osm_qos_policy_destroy(p_subn->p_qos_policy);
        p_subn->p_qos_policy = NULL;
        res = 1;
        goto Exit;
    }

    /* add generated ULP match rules to the usual match rules */
    __process_ulp_match_rules();

    if (osm_qos_policy_validate(p_subn->p_qos_policy,p_qos_parser_osm_log))
    {
        OSM_LOG(p_qos_parser_osm_log, OSM_LOG_ERROR, "ERR AC04: "
                "Error(s) in QoS policy file (%s)\n",
                p_subn->opt.qos_policy_file);
        fprintf(stderr, "Error(s) in QoS policy file (%s)\n",
                p_subn->opt.qos_policy_file);
        osm_qos_policy_destroy(p_subn->p_qos_policy);
        p_subn->p_qos_policy = NULL;
        res = 1;
        goto Exit;
    }

  Exit:
    if (yyin)
    {
        yyrestart(yyin);
        fclose(yyin);
    }
    OSM_LOG_EXIT(p_qos_parser_osm_log);
    return res;
}

/***************************************************
 ***************************************************/

int yywrap()
{
    return(1);
}

/***************************************************
 ***************************************************/

static void yyerror(const char *format, ...)
{
    char s[256];
    va_list pvar;

    OSM_LOG_ENTER(p_qos_parser_osm_log);

    va_start(pvar, format);
    vsnprintf(s, sizeof(s), format, pvar);
    va_end(pvar);

    OSM_LOG(p_qos_parser_osm_log, OSM_LOG_ERROR, "ERR AC05: "
            "Syntax error (line %d:%d): %s\n",
            line_num, column_num, s);
    fprintf(stderr, "Error in QoS Policy File (line %d:%d): %s.\n",
            line_num, column_num, s);
    OSM_LOG_EXIT(p_qos_parser_osm_log);
}

/***************************************************
 ***************************************************/

static char * __parser_strip_white(char * str)
{
	char *p;

	while (isspace(*str))
		str++;
	if (!*str)
		return str;
	p = str + strlen(str) - 1;
	while (isspace(*p))
		*p-- = '\0';

	return str;
}

/***************************************************
 ***************************************************/

static void __parser_str2uint64(uint64_t * p_val, char * str)
{
   *p_val = strtoull(str, NULL, 0);
}

/***************************************************
 ***************************************************/

static void __parser_port_group_start()
{
    p_current_port_group = osm_qos_policy_port_group_create();
}

/***************************************************
 ***************************************************/

static int __parser_port_group_end()
{
    if(!p_current_port_group->name)
    {
        yyerror("port-group validation failed - no port group name specified");
        return -1;
    }

    cl_list_insert_tail(&p_qos_policy->port_groups,
                        p_current_port_group);
    p_current_port_group = NULL;
    return 0;
}

/***************************************************
 ***************************************************/

static void __parser_vlarb_scope_start()
{
    p_current_vlarb_scope = osm_qos_policy_vlarb_scope_create();
}

/***************************************************
 ***************************************************/

static int __parser_vlarb_scope_end()
{
    if ( !cl_list_count(&p_current_vlarb_scope->group_list) &&
         !cl_list_count(&p_current_vlarb_scope->across_list) )
    {
        yyerror("vlarb-scope validation failed - no port groups specified by 'group' or by 'across'");
        return -1;
    }

    cl_list_insert_tail(&p_qos_policy->vlarb_tables,
                        p_current_vlarb_scope);
    p_current_vlarb_scope = NULL;
    return 0;
}

/***************************************************
 ***************************************************/

static void __parser_sl2vl_scope_start()
{
    p_current_sl2vl_scope = osm_qos_policy_sl2vl_scope_create();
}

/***************************************************
 ***************************************************/

static int __parser_sl2vl_scope_end()
{
    if (!p_current_sl2vl_scope->sl2vl_table_set)
    {
        yyerror("sl2vl-scope validation failed - no sl2vl table specified");
        return -1;
    }
    if ( !cl_list_count(&p_current_sl2vl_scope->group_list) &&
         !cl_list_count(&p_current_sl2vl_scope->across_to_list) &&
         !cl_list_count(&p_current_sl2vl_scope->across_from_list) )
    {
        yyerror("sl2vl-scope validation failed - no port groups specified by 'group', 'across-to' or 'across-from'");
        return -1;
    }

    cl_list_insert_tail(&p_qos_policy->sl2vl_tables,
                        p_current_sl2vl_scope);
    p_current_sl2vl_scope = NULL;
    return 0;
}

/***************************************************
 ***************************************************/

static void __parser_qos_level_start()
{
    p_current_qos_level = osm_qos_policy_qos_level_create();
}

/***************************************************
 ***************************************************/

static int __parser_qos_level_end()
{
    if (!p_current_qos_level->sl_set)
    {
        yyerror("qos-level validation failed - no 'sl' specified");
        return -1;
    }
    if (!p_current_qos_level->name)
    {
        yyerror("qos-level validation failed - no 'name' specified");
        return -1;
    }

    cl_list_insert_tail(&p_qos_policy->qos_levels,
                        p_current_qos_level);
    p_current_qos_level = NULL;
    return 0;
}

/***************************************************
 ***************************************************/

static void __parser_match_rule_start()
{
    p_current_qos_match_rule = osm_qos_policy_match_rule_create();
}

/***************************************************
 ***************************************************/

static int __parser_match_rule_end()
{
    if (!p_current_qos_match_rule->qos_level_name)
    {
        yyerror("match-rule validation failed - no 'qos-level-name' specified");
        return -1;
    }

    cl_list_insert_tail(&p_qos_policy->qos_match_rules,
                        p_current_qos_match_rule);
    p_current_qos_match_rule = NULL;
    return 0;
}

/***************************************************
 ***************************************************/

static void __parser_ulp_match_rule_start()
{
    p_current_qos_match_rule = osm_qos_policy_match_rule_create();
}

/***************************************************
 ***************************************************/

static int __parser_ulp_match_rule_end()
{
    CL_ASSERT(p_current_qos_match_rule->p_qos_level);
    cl_list_insert_tail(&__ulp_match_rules,
                        p_current_qos_match_rule);
    p_current_qos_match_rule = NULL;
    return 0;
}

/***************************************************
 ***************************************************/

static void __parser_tmp_struct_init()
{
    tmp_parser_struct.str[0] = '\0';
    cl_list_construct(&tmp_parser_struct.str_list);
    cl_list_init(&tmp_parser_struct.str_list, 10);
    cl_list_construct(&tmp_parser_struct.num_list);
    cl_list_init(&tmp_parser_struct.num_list, 10);
    cl_list_construct(&tmp_parser_struct.num_pair_list);
    cl_list_init(&tmp_parser_struct.num_pair_list, 10);
}

/***************************************************
 ***************************************************/

/*
 * Do NOT free objects from the temp struct.
 * Either they are inserted into the parse tree data
 * structure, or they are already freed when copying
 * their values to the parse tree data structure.
 */
static void __parser_tmp_struct_reset()
{
    tmp_parser_struct.str[0] = '\0';
    cl_list_remove_all(&tmp_parser_struct.str_list);
    cl_list_remove_all(&tmp_parser_struct.num_list);
    cl_list_remove_all(&tmp_parser_struct.num_pair_list);
}

/***************************************************
 ***************************************************/

static void __parser_tmp_struct_destroy()
{
    __parser_tmp_struct_reset();
    cl_list_destroy(&tmp_parser_struct.str_list);
    cl_list_destroy(&tmp_parser_struct.num_list);
    cl_list_destroy(&tmp_parser_struct.num_pair_list);
}

/***************************************************
 ***************************************************/

#define __SIMPLE_QOS_LEVEL_NAME "SimpleQoSLevel_SL"
#define __SIMPLE_QOS_LEVEL_DEFAULT_NAME "SimpleQoSLevel_DEFAULT"

static void __setup_simple_qos_levels()
{
    uint8_t i;
    char tmp_buf[30];
    memset(osm_qos_policy_simple_qos_levels, 0,
           sizeof(osm_qos_policy_simple_qos_levels));
    for (i = 0; i < 16; i++)
    {
        osm_qos_policy_simple_qos_levels[i].sl = i;
        osm_qos_policy_simple_qos_levels[i].sl_set = TRUE;
        sprintf(tmp_buf, "%s%u", __SIMPLE_QOS_LEVEL_NAME, i);
        osm_qos_policy_simple_qos_levels[i].name = strdup(tmp_buf);
    }

    memset(&__default_simple_qos_level, 0,
           sizeof(__default_simple_qos_level));
    __default_simple_qos_level.name =
           strdup(__SIMPLE_QOS_LEVEL_DEFAULT_NAME);
}

/***************************************************
 ***************************************************/

static void __clear_simple_qos_levels()
{
    /*
     * Simple QoS levels are static.
     * What's left is to invalidate default simple QoS level.
     */
    __default_simple_qos_level.sl_set = FALSE;
}

/***************************************************
 ***************************************************/

static void __setup_ulp_match_rules()
{
    cl_list_construct(&__ulp_match_rules);
    cl_list_init(&__ulp_match_rules, 10);
}

/***************************************************
 ***************************************************/

static void __process_ulp_match_rules()
{
    cl_list_iterator_t list_iterator;
    osm_qos_match_rule_t *p_qos_match_rule = NULL;

    list_iterator = cl_list_head(&__ulp_match_rules);
    while (list_iterator != cl_list_end(&__ulp_match_rules))
    {
        p_qos_match_rule = (osm_qos_match_rule_t *) cl_list_obj(list_iterator);
        if (p_qos_match_rule)
            cl_list_insert_tail(&p_qos_policy->qos_match_rules,
                                p_qos_match_rule);
        list_iterator = cl_list_next(list_iterator);
    }
    cl_list_remove_all(&__ulp_match_rules);
}

/***************************************************
 ***************************************************/

static int __cmp_num_range(const void * p1, const void * p2)
{
    uint64_t * pair1 = *((uint64_t **)p1);
    uint64_t * pair2 = *((uint64_t **)p2);

    if (pair1[0] < pair2[0])
        return -1;
    if (pair1[0] > pair2[0])
        return 1;

    if (pair1[1] < pair2[1])
        return -1;
    if (pair1[1] > pair2[1])
        return 1;

    return 0;
}

/***************************************************
 ***************************************************/

static void __sort_reduce_rangearr(
    uint64_t  **   arr,
    unsigned       arr_len,
    uint64_t  ** * p_res_arr,
    unsigned     * p_res_arr_len )
{
    unsigned i = 0;
    unsigned j = 0;
    unsigned last_valid_ind = 0;
    unsigned valid_cnt = 0;
    uint64_t ** res_arr;
    boolean_t * is_valid_arr;

    *p_res_arr = NULL;
    *p_res_arr_len = 0;

    qsort(arr, arr_len, sizeof(uint64_t*), __cmp_num_range);

    is_valid_arr = (boolean_t *)malloc(arr_len * sizeof(boolean_t));
    is_valid_arr[last_valid_ind] = TRUE;
    valid_cnt++;
    for (i = 1; i < arr_len; i++)
    {
        if (arr[i][0] <= arr[last_valid_ind][1])
        {
            if (arr[i][1] > arr[last_valid_ind][1])
                arr[last_valid_ind][1] = arr[i][1];
            free(arr[i]);
            arr[i] = NULL;
            is_valid_arr[i] = FALSE;
        }
        else if ((arr[i][0] - 1) == arr[last_valid_ind][1])
        {
            arr[last_valid_ind][1] = arr[i][1];
            free(arr[i]);
            arr[i] = NULL;
            is_valid_arr[i] = FALSE;
        }
        else
        {
            is_valid_arr[i] = TRUE;
            last_valid_ind = i;
            valid_cnt++;
        }
    }

    res_arr = (uint64_t **)malloc(valid_cnt * sizeof(uint64_t *));
    for (i = 0; i < arr_len; i++)
    {
        if (is_valid_arr[i])
            res_arr[j++] = arr[i];
    }
    free(is_valid_arr);
    free(arr);

    *p_res_arr = res_arr;
    *p_res_arr_len = valid_cnt;
}

/***************************************************
 ***************************************************/

static void __pkey_rangelist2rangearr(
    cl_list_t    * p_list,
    uint64_t  ** * p_arr,
    unsigned     * p_arr_len)
{
    uint64_t   tmp_pkey;
    uint64_t * p_pkeys;
    cl_list_iterator_t list_iterator;

    list_iterator= cl_list_head(p_list);
    while( list_iterator != cl_list_end(p_list) )
    {
       p_pkeys = (uint64_t *)cl_list_obj(list_iterator);
       p_pkeys[0] &= 0x7fff;
       p_pkeys[1] &= 0x7fff;
       if (p_pkeys[0] > p_pkeys[1])
       {
           tmp_pkey = p_pkeys[1];
           p_pkeys[1] = p_pkeys[0];
           p_pkeys[0] = tmp_pkey;
       }
       list_iterator = cl_list_next(list_iterator);
    }

    __rangelist2rangearr(p_list, p_arr, p_arr_len);
}

/***************************************************
 ***************************************************/

static void __rangelist2rangearr(
    cl_list_t    * p_list,
    uint64_t  ** * p_arr,
    unsigned     * p_arr_len)
{
    cl_list_iterator_t list_iterator;
    unsigned len = cl_list_count(p_list);
    unsigned i = 0;
    uint64_t ** tmp_arr;
    uint64_t ** res_arr = NULL;
    unsigned res_arr_len = 0;

    tmp_arr = (uint64_t **)malloc(len * sizeof(uint64_t *));

    list_iterator = cl_list_head(p_list);
    while( list_iterator != cl_list_end(p_list) )
    {
       tmp_arr[i++] = (uint64_t *)cl_list_obj(list_iterator);
       list_iterator = cl_list_next(list_iterator);
    }
    cl_list_remove_all(p_list);

    __sort_reduce_rangearr( tmp_arr,
                            len,
                            &res_arr,
                            &res_arr_len );
    *p_arr = res_arr;
    *p_arr_len = res_arr_len;
}

/***************************************************
 ***************************************************/

static void __merge_rangearr(
    uint64_t  **   range_arr_1,
    unsigned       range_len_1,
    uint64_t  **   range_arr_2,
    unsigned       range_len_2,
    uint64_t  ** * p_arr,
    unsigned     * p_arr_len )
{
    unsigned i = 0;
    unsigned j = 0;
    unsigned len = range_len_1 + range_len_2;
    uint64_t ** tmp_arr;
    uint64_t ** res_arr = NULL;
    unsigned res_arr_len = 0;

    *p_arr = NULL;
    *p_arr_len = 0;

    tmp_arr = (uint64_t **)malloc(len * sizeof(uint64_t *));

    for (i = 0; i < range_len_1; i++)
       tmp_arr[j++] = range_arr_1[i];
    for (i = 0; i < range_len_2; i++)
       tmp_arr[j++] = range_arr_2[i];
    free(range_arr_1);
    free(range_arr_2);

    __sort_reduce_rangearr( tmp_arr,
                            len,
                            &res_arr,
                            &res_arr_len );
    *p_arr = res_arr;
    *p_arr_len = res_arr_len;
}

/***************************************************
 ***************************************************/

static void __parser_add_port_to_port_map(
    cl_qmap_t   * p_map,
    osm_physp_t * p_physp)
{
    if (cl_qmap_get(p_map, cl_ntoh64(osm_physp_get_port_guid(p_physp))) ==
        cl_qmap_end(p_map))
    {
        osm_qos_port_t * p_port = osm_qos_policy_port_create(p_physp);
        if (p_port)
            cl_qmap_insert(p_map,
                           cl_ntoh64(osm_physp_get_port_guid(p_physp)),
                           &p_port->map_item);
    }
}

/***************************************************
 ***************************************************/

static void __parser_add_guid_range_to_port_map(
    cl_qmap_t  * p_map,
    uint64_t  ** range_arr,
    unsigned     range_len)
{
    unsigned i;
    uint64_t guid_ho;
    osm_port_t * p_osm_port;

    if (!range_arr || !range_len)
        return;

    for (i = 0; i < range_len; i++) {
         for (guid_ho = range_arr[i][0]; guid_ho <= range_arr[i][1]; guid_ho++) {
             p_osm_port =
                osm_get_port_by_guid(p_qos_policy->p_subn, cl_hton64(guid_ho));
             if (p_osm_port)
                 __parser_add_port_to_port_map(p_map, p_osm_port->p_physp);
         }
         free(range_arr[i]);
    }
    free(range_arr);
}

/***************************************************
 ***************************************************/

static void __parser_add_pkey_range_to_port_map(
    cl_qmap_t  * p_map,
    uint64_t  ** range_arr,
    unsigned     range_len)
{
    unsigned i;
    uint64_t pkey_64;
    ib_net16_t pkey;
    osm_prtn_t * p_prtn;

    if (!range_arr || !range_len)
        return;

    for (i = 0; i < range_len; i++) {
         for (pkey_64 = range_arr[i][0]; pkey_64 <= range_arr[i][1]; pkey_64++) {
             pkey = cl_hton16((uint16_t)(pkey_64 & 0x7fff));
             p_prtn = (osm_prtn_t *)
                 cl_qmap_get(&p_qos_policy->p_subn->prtn_pkey_tbl, pkey);
             if (p_prtn != (osm_prtn_t *)cl_qmap_end(
                   &p_qos_policy->p_subn->prtn_pkey_tbl)) {
                 __parser_add_map_to_port_map(p_map, &p_prtn->part_guid_tbl);
                 __parser_add_map_to_port_map(p_map, &p_prtn->full_guid_tbl);
             }
         }
         free(range_arr[i]);
    }
    free(range_arr);
}

/***************************************************
 ***************************************************/

static void __parser_add_partition_list_to_port_map(
    cl_qmap_t  * p_map,
    cl_list_t  * p_list)
{
    cl_list_iterator_t    list_iterator;
    char                * tmp_str;
    osm_prtn_t          * p_prtn;

    /* extract all the ports from the partition
       to the port map of this port group */
    list_iterator = cl_list_head(p_list);
    while(list_iterator != cl_list_end(p_list)) {
        tmp_str = (char*)cl_list_obj(list_iterator);
        if (tmp_str) {
            p_prtn = osm_prtn_find_by_name(p_qos_policy->p_subn, tmp_str);
            if (p_prtn) {
                __parser_add_map_to_port_map(p_map, &p_prtn->part_guid_tbl);
                __parser_add_map_to_port_map(p_map, &p_prtn->full_guid_tbl);
            }
            free(tmp_str);
        }
        list_iterator = cl_list_next(list_iterator);
    }
    cl_list_remove_all(p_list);
}

/***************************************************
 ***************************************************/

static void __parser_add_map_to_port_map(
    cl_qmap_t * p_dmap,
    cl_map_t  * p_smap)
{
    cl_map_iterator_t map_iterator;
    osm_physp_t * p_physp;

    if (!p_dmap || !p_smap)
        return;

    map_iterator = cl_map_head(p_smap);
    while (map_iterator != cl_map_end(p_smap)) {
        p_physp = (osm_physp_t*)cl_map_obj(map_iterator);
        __parser_add_port_to_port_map(p_dmap, p_physp);
        map_iterator = cl_map_next(map_iterator);
    }
}

/***************************************************
 ***************************************************/

static int __validate_pkeys( uint64_t ** range_arr,
                             unsigned    range_len,
                             boolean_t   is_ipoib)
{
    unsigned i;
    uint64_t pkey_64;
    ib_net16_t pkey;
    osm_prtn_t * p_prtn;

    if (!range_arr || !range_len)
        return 0;

    for (i = 0; i < range_len; i++) {
        for (pkey_64 = range_arr[i][0]; pkey_64 <= range_arr[i][1]; pkey_64++) {
            pkey = cl_hton16((uint16_t)(pkey_64 & 0x7fff));
            p_prtn = (osm_prtn_t *)
                cl_qmap_get(&p_qos_policy->p_subn->prtn_pkey_tbl, pkey);

            if (p_prtn == (osm_prtn_t *)cl_qmap_end(
                  &p_qos_policy->p_subn->prtn_pkey_tbl))
                p_prtn = NULL;

            if (is_ipoib) {
                /*
                 * Be very strict for IPoIB partition:
                 *  - the partition for the pkey have to exist
                 *  - it has to have at least 2 full members
                 */
                if (!p_prtn) {
                    yyerror("IPoIB partition, pkey 0x%04X - "
                                       "partition doesn't exist",
                                       cl_ntoh16(pkey));
                    return 1;
                }
                else if (cl_map_count(&p_prtn->full_guid_tbl) < 2) {
                    yyerror("IPoIB partition, pkey 0x%04X - "
                                       "partition has less than two full members",
                                       cl_ntoh16(pkey));
                    return 1;
                }
            }
            else if (!p_prtn) {
                /*
                 * For non-IPoIB pkey we just want to check that
                 * the relevant partition exists.
                 * And even if it doesn't, don't exit - just print
                 * error message and continue.
                 */
                 OSM_LOG(p_qos_parser_osm_log, OSM_LOG_ERROR, "ERR AC02: "
			 "pkey 0x%04X - partition doesn't exist",
                         cl_ntoh16(pkey));
            }
        }
    }
    return 0;
}

/***************************************************
 ***************************************************/
#line 1764 "osm_qos_parser_y.c"

#if YYDEBUG
#include <stdio.h>	/* needed for printf */
#endif

#include <stdlib.h>	/* needed for malloc, etc */
#include <string.h>	/* needed for memset */

/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(YYSTACKDATA *data)
{
    int i;
    unsigned newsize;
    YYINT *newss;
    YYSTYPE *newvs;

    if ((newsize = data->stacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return YYENOMEM;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = (int) (data->s_mark - data->s_base);
    newss = (YYINT *)realloc(data->s_base, newsize * sizeof(*newss));
    if (newss == 0)
        return YYENOMEM;

    data->s_base = newss;
    data->s_mark = newss + i;

    newvs = (YYSTYPE *)realloc(data->l_base, newsize * sizeof(*newvs));
    if (newvs == 0)
        return YYENOMEM;

    data->l_base = newvs;
    data->l_mark = newvs + i;

    data->stacksize = newsize;
    data->s_last = data->s_base + newsize - 1;
    return 0;
}

#if YYPURE || defined(YY_NO_LEAKS)
static void yyfreestack(YYSTACKDATA *data)
{
    free(data->s_base);
    free(data->l_base);
    memset(data, 0, sizeof(*data));
}
#else
#define yyfreestack(data) /* nothing */
#endif

#define YYABORT  goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab

int
YYPARSE_DECL()
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    /* yym is set below */
    /* yyn is set below */
    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;
    yystate = 0;

#if YYPURE
    memset(&yystack, 0, sizeof(yystack));
#endif

    if (yystack.s_base == NULL && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    yystack.s_mark = yystack.s_base;
    yystack.l_mark = yystack.l_base;
    yystate = 0;
    *yystack.s_mark = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        yychar = YYLEX;
        if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if (((yyn = yysindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
        yystate = yytable[yyn];
        *++yystack.s_mark = yytable[yyn];
        *++yystack.l_mark = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if (((yyn = yyrindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag != 0) goto yyinrecovery;

    YYERROR_CALL("syntax error");

    goto yyerrlab; /* redundant goto avoids 'unused label' warning */
yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if (((yyn = yysindex[*yystack.s_mark]) != 0) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yystack.s_mark, yytable[yyn]);
#endif
                if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
                yystate = yytable[yyn];
                *++yystack.s_mark = yytable[yyn];
                *++yystack.l_mark = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yystack.s_mark);
#endif
                if (yystack.s_mark <= yystack.s_base) goto yyabort;
                --yystack.s_mark;
                --yystack.l_mark;
            }
        }
    }
    else
    {
        if (yychar == YYEOF) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym > 0)
        yyval = yystack.l_mark[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);

    switch (yyn)
    {
case 16:
#line 365 "osm_qos_parser_y.y"
	{
                        __parser_port_group_start();
                    }
#line 1968 "osm_qos_parser_y.c"
break;
case 17:
#line 370 "osm_qos_parser_y.y"
	{
                        if ( __parser_port_group_end() )
                            return 1;
                    }
#line 1976 "osm_qos_parser_y.c"
break;
case 35:
#line 433 "osm_qos_parser_y.y"
	{
                        __parser_vlarb_scope_start();
                    }
#line 1983 "osm_qos_parser_y.c"
break;
case 36:
#line 438 "osm_qos_parser_y.y"
	{
                        if ( __parser_vlarb_scope_end() )
                            return 1;
                    }
#line 1991 "osm_qos_parser_y.c"
break;
case 48:
#line 479 "osm_qos_parser_y.y"
	{
                        __parser_sl2vl_scope_start();
                    }
#line 1998 "osm_qos_parser_y.c"
break;
case 49:
#line 484 "osm_qos_parser_y.y"
	{
                        if ( __parser_sl2vl_scope_end() )
                            return 1;
                    }
#line 2006 "osm_qos_parser_y.c"
break;
case 63:
#line 550 "osm_qos_parser_y.y"
	{
                        __parser_qos_level_start();
                    }
#line 2013 "osm_qos_parser_y.c"
break;
case 64:
#line 555 "osm_qos_parser_y.y"
	{
                        if ( __parser_qos_level_end() )
                            return 1;
                    }
#line 2021 "osm_qos_parser_y.c"
break;
case 79:
#line 604 "osm_qos_parser_y.y"
	{
                        __parser_match_rule_start();
                    }
#line 2028 "osm_qos_parser_y.c"
break;
case 80:
#line 609 "osm_qos_parser_y.y"
	{
                        if ( __parser_match_rule_end() )
                            return 1;
                    }
#line 2036 "osm_qos_parser_y.c"
break;
case 90:
#line 649 "osm_qos_parser_y.y"
	{
                        /* parsing default ulp rule: "default: num" */
                        cl_list_iterator_t    list_iterator;
                        uint64_t            * p_tmp_num;

                        list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                        p_tmp_num = (uint64_t*)cl_list_obj(list_iterator);
                        if (*p_tmp_num > 15)
                        {
                            yyerror("illegal SL value");
                            return 1;
                        }
                        __default_simple_qos_level.sl = (uint8_t)(*p_tmp_num);
                        __default_simple_qos_level.sl_set = TRUE;
                        free(p_tmp_num);
                        cl_list_remove_all(&tmp_parser_struct.num_list);
                    }
#line 2057 "osm_qos_parser_y.c"
break;
case 91:
#line 667 "osm_qos_parser_y.y"
	{
                        /* "any, service-id ... : sl" - one instance of list of ranges */
                        uint64_t ** range_arr;
                        unsigned    range_len;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("ULP rule doesn't have service ids");
                            return 1;
                        }

                        /* get all the service id ranges */
                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );

                        p_current_qos_match_rule->service_id_range_arr = range_arr;
                        p_current_qos_match_rule->service_id_range_len = range_len;

                    }
#line 2081 "osm_qos_parser_y.c"
break;
case 93:
#line 688 "osm_qos_parser_y.y"
	{
                        /* "any, pkey ... : sl" - one instance of list of ranges */
                        uint64_t ** range_arr;
                        unsigned    range_len;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("ULP rule doesn't have pkeys");
                            return 1;
                        }

                        /* get all the pkey ranges */
                        __pkey_rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );

                        p_current_qos_match_rule->pkey_range_arr = range_arr;
                        p_current_qos_match_rule->pkey_range_len = range_len;

                    }
#line 2105 "osm_qos_parser_y.c"
break;
case 95:
#line 709 "osm_qos_parser_y.y"
	{
                        /* any, target-port-guid ... : sl */
                        uint64_t ** range_arr;
                        unsigned    range_len;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("ULP rule doesn't have port guids");
                            return 1;
                        }

                        /* create a new port group with these ports */
                        __parser_port_group_start();

                        p_current_port_group->name = strdup("_ULP_Targets_");
                        p_current_port_group->use = strdup("Generated from ULP rules");

                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );

                        __parser_add_guid_range_to_port_map(
                                              &p_current_port_group->port_map,
                                              range_arr,
                                              range_len);

                        /* add this port group to the destination
                           groups of the current match rule */
                        cl_list_insert_tail(&p_current_qos_match_rule->destination_group_list,
                                            p_current_port_group);

                        __parser_port_group_end();

                    }
#line 2143 "osm_qos_parser_y.c"
break;
case 97:
#line 744 "osm_qos_parser_y.y"
	{
			/* any, source-port-guid ... : sl */
			uint64_t ** range_arr;
			unsigned    range_len;

			if (!cl_list_count(&tmp_parser_struct.num_pair_list))
			{
				yyerror("ULP rule doesn't have port guids");
				return 1;
			}

                        /* create a new port group with these ports */
                        __parser_port_group_start();

                        p_current_port_group->name = strdup("_ULP_Sources_");
                        p_current_port_group->use = strdup("Generated from ULP rules");

                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );

                        __parser_add_guid_range_to_port_map(
                                              &p_current_port_group->port_map,
                                              range_arr,
                                              range_len);

                        /* add this port group to the source
                           groups of the current match rule */
                        cl_list_insert_tail(&p_current_qos_match_rule->source_group_list,
                                            p_current_port_group);

                        __parser_port_group_end();

		    }
#line 2181 "osm_qos_parser_y.c"
break;
case 99:
#line 779 "osm_qos_parser_y.y"
	{
			/* any, source-target-port-guid ... : sl */
			uint64_t ** range_arr;
			unsigned    range_len;

			if (!cl_list_count(&tmp_parser_struct.num_pair_list))
			{
				yyerror("ULP rule doesn't have port guids");
				return 1;
			}

                        /* create a new port group with these ports */
                        __parser_port_group_start();

                        p_current_port_group->name = strdup("_ULP_Sources_Targets_");
                        p_current_port_group->use = strdup("Generated from ULP rules");

                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );

                        __parser_add_guid_range_to_port_map(
                                              &p_current_port_group->port_map,
                                              range_arr,
                                              range_len);

                        /* add this port group to the source and destination
                           groups of the current match rule */
                        cl_list_insert_tail(&p_current_qos_match_rule->source_group_list,
                                            p_current_port_group);

                        cl_list_insert_tail(&p_current_qos_match_rule->destination_group_list,
                                            p_current_port_group);

                        __parser_port_group_end();

		    }
#line 2222 "osm_qos_parser_y.c"
break;
case 101:
#line 817 "osm_qos_parser_y.y"
	{
                        /* "sdp : sl" - default SL for SDP */
                        uint64_t ** range_arr =
                               (uint64_t **)malloc(sizeof(uint64_t *));
                        range_arr[0] = (uint64_t *)malloc(2*sizeof(uint64_t));
                        range_arr[0][0] = OSM_QOS_POLICY_ULP_SDP_SERVICE_ID;
                        range_arr[0][1] = OSM_QOS_POLICY_ULP_SDP_SERVICE_ID + 0xFFFF;

                        p_current_qos_match_rule->service_id_range_arr = range_arr;
                        p_current_qos_match_rule->service_id_range_len = 1;

                    }
#line 2238 "osm_qos_parser_y.c"
break;
case 103:
#line 830 "osm_qos_parser_y.y"
	{
                        /* sdp with port numbers */
                        uint64_t ** range_arr;
                        unsigned    range_len;
                        unsigned    i;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("SDP ULP rule doesn't have port numbers");
                            return 1;
                        }

                        /* get all the port ranges */
                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );
                        /* now translate these port numbers into service ids */
                        for (i = 0; i < range_len; i++)
                        {
                            if (range_arr[i][0] > 0xFFFF || range_arr[i][1] > 0xFFFF)
                            {
                                yyerror("SDP port number out of range");
				free(range_arr);
                                return 1;
                            }
                            range_arr[i][0] += OSM_QOS_POLICY_ULP_SDP_SERVICE_ID;
                            range_arr[i][1] += OSM_QOS_POLICY_ULP_SDP_SERVICE_ID;
                        }

                        p_current_qos_match_rule->service_id_range_arr = range_arr;
                        p_current_qos_match_rule->service_id_range_len = range_len;

                    }
#line 2275 "osm_qos_parser_y.c"
break;
case 105:
#line 864 "osm_qos_parser_y.y"
	{
                        /* "rds : sl" - default SL for RDS */
                        uint64_t ** range_arr =
                               (uint64_t **)malloc(sizeof(uint64_t *));
                        range_arr[0] = (uint64_t *)malloc(2*sizeof(uint64_t));
                        range_arr[0][0] = range_arr[0][1] =
                           OSM_QOS_POLICY_ULP_RDS_SERVICE_ID + OSM_QOS_POLICY_ULP_RDS_PORT;

                        p_current_qos_match_rule->service_id_range_arr = range_arr;
                        p_current_qos_match_rule->service_id_range_len = 1;

                    }
#line 2291 "osm_qos_parser_y.c"
break;
case 107:
#line 877 "osm_qos_parser_y.y"
	{
                        /* rds with port numbers */
                        uint64_t ** range_arr;
                        unsigned    range_len;
                        unsigned    i;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("RDS ULP rule doesn't have port numbers");
                            return 1;
                        }

                        /* get all the port ranges */
                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );
                        /* now translate these port numbers into service ids */
                        for (i = 0; i < range_len; i++)
                        {
                            if (range_arr[i][0] > 0xFFFF || range_arr[i][1] > 0xFFFF)
                            {
                                yyerror("SDP port number out of range");
				free(range_arr);
                                return 1;
                            }
                            range_arr[i][0] += OSM_QOS_POLICY_ULP_RDS_SERVICE_ID;
                            range_arr[i][1] += OSM_QOS_POLICY_ULP_RDS_SERVICE_ID;
                        }

                        p_current_qos_match_rule->service_id_range_arr = range_arr;
                        p_current_qos_match_rule->service_id_range_len = range_len;

                    }
#line 2328 "osm_qos_parser_y.c"
break;
case 109:
#line 911 "osm_qos_parser_y.y"
	{
                        /* "iSER : sl" - default SL for iSER */
                        uint64_t ** range_arr =
                               (uint64_t **)malloc(sizeof(uint64_t *));
                        range_arr[0] = (uint64_t *)malloc(2*sizeof(uint64_t));
                        range_arr[0][0] = range_arr[0][1] =
                           OSM_QOS_POLICY_ULP_ISER_SERVICE_ID + OSM_QOS_POLICY_ULP_ISER_PORT;

                        p_current_qos_match_rule->service_id_range_arr = range_arr;
                        p_current_qos_match_rule->service_id_range_len = 1;

                    }
#line 2344 "osm_qos_parser_y.c"
break;
case 111:
#line 924 "osm_qos_parser_y.y"
	{
                        /* iser with port numbers */
                        uint64_t ** range_arr;
                        unsigned    range_len;
                        unsigned    i;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("iSER ULP rule doesn't have port numbers");
                            return 1;
                        }

                        /* get all the port ranges */
                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );
                        /* now translate these port numbers into service ids */
                        for (i = 0; i < range_len; i++)
                        {
                            if (range_arr[i][0] > 0xFFFF || range_arr[i][1] > 0xFFFF)
                            {
                                yyerror("SDP port number out of range");
				free(range_arr);
                                return 1;
                            }
                            range_arr[i][0] += OSM_QOS_POLICY_ULP_ISER_SERVICE_ID;
                            range_arr[i][1] += OSM_QOS_POLICY_ULP_ISER_SERVICE_ID;
                        }

                        p_current_qos_match_rule->service_id_range_arr = range_arr;
                        p_current_qos_match_rule->service_id_range_len = range_len;

                    }
#line 2381 "osm_qos_parser_y.c"
break;
case 113:
#line 958 "osm_qos_parser_y.y"
	{
                        /* srp with target guids - this rule is similar
                           to writing 'any' ulp with target port guids */
                        uint64_t ** range_arr;
                        unsigned    range_len;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("SRP ULP rule doesn't have port guids");
                            return 1;
                        }

                        /* create a new port group with these ports */
                        __parser_port_group_start();

                        p_current_port_group->name = strdup("_SRP_Targets_");
                        p_current_port_group->use = strdup("Generated from ULP rules");

                        __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );

                        __parser_add_guid_range_to_port_map(
                                              &p_current_port_group->port_map,
                                              range_arr,
                                              range_len);

                        /* add this port group to the destination
                           groups of the current match rule */
                        cl_list_insert_tail(&p_current_qos_match_rule->destination_group_list,
                                            p_current_port_group);

                        __parser_port_group_end();

                    }
#line 2420 "osm_qos_parser_y.c"
break;
case 115:
#line 994 "osm_qos_parser_y.y"
	{
                        /* ipoib w/o any pkeys (default pkey) */
                        uint64_t ** range_arr =
                               (uint64_t **)malloc(sizeof(uint64_t *));
                        range_arr[0] = (uint64_t *)malloc(2*sizeof(uint64_t));
                        range_arr[0][0] = range_arr[0][1] = 0x7fff;

                        /*
                         * Although we know that the default partition exists,
                         * we still need to validate it by checking that it has
                         * at least two full members. Otherwise IPoIB won't work.
                         */
                        if (__validate_pkeys(range_arr, 1, TRUE)) {
                            free(range_arr[0]);
                            free(range_arr);
                            return 1;
			}

                        p_current_qos_match_rule->pkey_range_arr = range_arr;
                        p_current_qos_match_rule->pkey_range_len = 1;

                    }
#line 2446 "osm_qos_parser_y.c"
break;
case 117:
#line 1017 "osm_qos_parser_y.y"
	{
                        /* ipoib with pkeys */
                        uint64_t ** range_arr;
                        unsigned    range_len;

                        if (!cl_list_count(&tmp_parser_struct.num_pair_list))
                        {
                            yyerror("IPoIB ULP rule doesn't have pkeys");
                            return 1;
                        }

                        /* get all the pkey ranges */
                        __pkey_rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                              &range_arr,
                                              &range_len );

                        /*
                         * Validate pkeys.
                         * For IPoIB pkeys the validation is strict.
                         * If some problem would be found, parsing will
                         * be aborted with a proper error messages.
                         */
			if (__validate_pkeys(range_arr, range_len, TRUE)) {
			    free(range_arr);
                            return 1;
			}

                        p_current_qos_match_rule->pkey_range_arr = range_arr;
                        p_current_qos_match_rule->pkey_range_len = range_len;

                    }
#line 2481 "osm_qos_parser_y.c"
break;
case 119:
#line 1051 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2486 "osm_qos_parser_y.c"
break;
case 120:
#line 1054 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2491 "osm_qos_parser_y.c"
break;
case 121:
#line 1057 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2496 "osm_qos_parser_y.c"
break;
case 122:
#line 1060 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2501 "osm_qos_parser_y.c"
break;
case 123:
#line 1063 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2506 "osm_qos_parser_y.c"
break;
case 124:
#line 1066 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2511 "osm_qos_parser_y.c"
break;
case 125:
#line 1069 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2516 "osm_qos_parser_y.c"
break;
case 126:
#line 1072 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2521 "osm_qos_parser_y.c"
break;
case 127:
#line 1075 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2526 "osm_qos_parser_y.c"
break;
case 128:
#line 1078 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2531 "osm_qos_parser_y.c"
break;
case 129:
#line 1081 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2536 "osm_qos_parser_y.c"
break;
case 130:
#line 1084 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2541 "osm_qos_parser_y.c"
break;
case 131:
#line 1087 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2546 "osm_qos_parser_y.c"
break;
case 132:
#line 1090 "osm_qos_parser_y.y"
	{ __parser_ulp_match_rule_start(); }
#line 2551 "osm_qos_parser_y.c"
break;
case 133:
#line 1093 "osm_qos_parser_y.y"
	{
                        /* get the SL for ULP rules */
                        cl_list_iterator_t  list_iterator;
                        uint64_t          * p_tmp_num;
                        uint8_t             sl;

                        list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                        p_tmp_num = (uint64_t*)cl_list_obj(list_iterator);
                        if (*p_tmp_num > 15)
                        {
                            yyerror("illegal SL value");
                            return 1;
                        }

                        sl = (uint8_t)(*p_tmp_num);
                        free(p_tmp_num);
                        cl_list_remove_all(&tmp_parser_struct.num_list);

                        p_current_qos_match_rule->p_qos_level =
                                 &osm_qos_policy_simple_qos_levels[sl];
                        p_current_qos_match_rule->qos_level_name =
                                 strdup(osm_qos_policy_simple_qos_levels[sl].name);

                        if (__parser_ulp_match_rule_end())
                            return 1;
                    }
#line 2581 "osm_qos_parser_y.c"
break;
case 134:
#line 1132 "osm_qos_parser_y.y"
	{
                            /* 'name' of 'port-group' - one instance */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            if (p_current_port_group->name)
                            {
                                yyerror("port-group has multiple 'name' tags");
                                cl_list_remove_all(&tmp_parser_struct.str_list);
                                return 1;
                            }

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            if ( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    p_current_port_group->name = tmp_str;
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 2606 "osm_qos_parser_y.c"
break;
case 135:
#line 1155 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2613 "osm_qos_parser_y.c"
break;
case 136:
#line 1160 "osm_qos_parser_y.y"
	{
                            /* 'use' of 'port-group' - one instance */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            if (p_current_port_group->use)
                            {
                                yyerror("port-group has multiple 'use' tags");
                                cl_list_remove_all(&tmp_parser_struct.str_list);
                                return 1;
                            }

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            if ( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    p_current_port_group->use = tmp_str;
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 2638 "osm_qos_parser_y.c"
break;
case 137:
#line 1183 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2645 "osm_qos_parser_y.c"
break;
case 138:
#line 1188 "osm_qos_parser_y.y"
	{
                            /* 'port-name' in 'port-group' - any num of instances */
                            cl_list_iterator_t list_iterator;
                            osm_node_t * p_node;
                            osm_physp_t * p_physp;
                            unsigned port_num;
                            char * tmp_str;
                            char * port_str;

                            /* parsing port name strings */
                            for (list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                                 list_iterator != cl_list_end(&tmp_parser_struct.str_list);
                                 list_iterator = cl_list_next(list_iterator))
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                {
                                    /* last slash in port name string is a separator
                                       between node name and port number */
                                    port_str = strrchr(tmp_str, '/');
                                    if (!port_str || (strlen(port_str) < 3) ||
                                        (port_str[1] != 'p' && port_str[1] != 'P')) {
                                        yyerror("'%s' - illegal port name",
                                                           tmp_str);
                                        free(tmp_str);
                                        cl_list_remove_all(&tmp_parser_struct.str_list);
                                        return 1;
                                    }

                                    if (!(port_num = strtoul(&port_str[2],NULL,0))) {
                                        yyerror(
                                               "'%s' - illegal port number in port name",
                                               tmp_str);
                                        free(tmp_str);
                                        cl_list_remove_all(&tmp_parser_struct.str_list);
                                        return 1;
                                    }

                                    /* separate node name from port number */
                                    port_str[0] = '\0';

                                    if (st_lookup(p_qos_policy->p_node_hash,
                                                  (st_data_t)tmp_str,
                                                  (void *)&p_node))
                                    {
                                        /* we found the node, now get the right port */
                                        p_physp = osm_node_get_physp_ptr(p_node, port_num);
                                        if (!p_physp) {
                                            yyerror(
                                                   "'%s' - port number out of range in port name",
                                                   tmp_str);
                                            free(tmp_str);
                                            cl_list_remove_all(&tmp_parser_struct.str_list);
                                            return 1;
                                        }
                                        /* we found the port, now add it to guid table */
                                        __parser_add_port_to_port_map(&p_current_port_group->port_map,
                                                                      p_physp);
                                    }
                                    free(tmp_str);
                                }
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 2713 "osm_qos_parser_y.c"
break;
case 139:
#line 1254 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2720 "osm_qos_parser_y.c"
break;
case 140:
#line 1259 "osm_qos_parser_y.y"
	{
                            /* 'port-guid' in 'port-group' - any num of instances */
                            /* list of guid ranges */
                            if (cl_list_count(&tmp_parser_struct.num_pair_list))
                            {
                                uint64_t ** range_arr;
                                unsigned range_len;

                                __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                                      &range_arr,
                                                      &range_len );

                                __parser_add_guid_range_to_port_map(
                                                      &p_current_port_group->port_map,
                                                      range_arr,
                                                      range_len);
                            }
                        }
#line 2742 "osm_qos_parser_y.c"
break;
case 141:
#line 1279 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2749 "osm_qos_parser_y.c"
break;
case 142:
#line 1284 "osm_qos_parser_y.y"
	{
                            /* 'pkey' in 'port-group' - any num of instances */
                            /* list of pkey ranges */
                            if (cl_list_count(&tmp_parser_struct.num_pair_list))
                            {
                                uint64_t ** range_arr;
                                unsigned range_len;

                                __pkey_rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                                      &range_arr,
                                                      &range_len );

                                __parser_add_pkey_range_to_port_map(
                                                      &p_current_port_group->port_map,
                                                      range_arr,
                                                      range_len);
                            }
                        }
#line 2771 "osm_qos_parser_y.c"
break;
case 143:
#line 1304 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2778 "osm_qos_parser_y.c"
break;
case 144:
#line 1309 "osm_qos_parser_y.y"
	{
                            /* 'partition' in 'port-group' - any num of instances */
                            __parser_add_partition_list_to_port_map(
                                               &p_current_port_group->port_map,
                                               &tmp_parser_struct.str_list);
                        }
#line 2788 "osm_qos_parser_y.c"
break;
case 145:
#line 1317 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2795 "osm_qos_parser_y.c"
break;
case 146:
#line 1322 "osm_qos_parser_y.y"
	{
                            /* 'node-type' in 'port-group' - any num of instances */
                        }
#line 2802 "osm_qos_parser_y.c"
break;
case 147:
#line 1327 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2809 "osm_qos_parser_y.c"
break;
case 155:
#line 1343 "osm_qos_parser_y.y"
	{
                            p_current_port_group->node_types |=
                               OSM_QOS_POLICY_NODE_TYPE_CA;
                        }
#line 2817 "osm_qos_parser_y.c"
break;
case 156:
#line 1349 "osm_qos_parser_y.y"
	{
                            p_current_port_group->node_types |=
                               OSM_QOS_POLICY_NODE_TYPE_SWITCH;
                        }
#line 2825 "osm_qos_parser_y.c"
break;
case 157:
#line 1355 "osm_qos_parser_y.y"
	{
                            p_current_port_group->node_types |=
                               OSM_QOS_POLICY_NODE_TYPE_ROUTER;
                        }
#line 2833 "osm_qos_parser_y.c"
break;
case 158:
#line 1361 "osm_qos_parser_y.y"
	{
                            p_current_port_group->node_types |=
                               (OSM_QOS_POLICY_NODE_TYPE_CA |
                                OSM_QOS_POLICY_NODE_TYPE_SWITCH |
                                OSM_QOS_POLICY_NODE_TYPE_ROUTER);
                        }
#line 2843 "osm_qos_parser_y.c"
break;
case 159:
#line 1369 "osm_qos_parser_y.y"
	{
                            osm_port_t * p_osm_port =
                                osm_get_port_by_guid(p_qos_policy->p_subn,
                                     p_qos_policy->p_subn->sm_port_guid);
                            if (p_osm_port)
                                __parser_add_port_to_port_map(
                                   &p_current_port_group->port_map,
                                   p_osm_port->p_physp);
                        }
#line 2856 "osm_qos_parser_y.c"
break;
case 160:
#line 1391 "osm_qos_parser_y.y"
	{
                            /* 'group' in 'vlarb-scope' - any num of instances */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    cl_list_insert_tail(&p_current_vlarb_scope->group_list,tmp_str);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 2875 "osm_qos_parser_y.c"
break;
case 161:
#line 1408 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2882 "osm_qos_parser_y.c"
break;
case 162:
#line 1413 "osm_qos_parser_y.y"
	{
                            /* 'across' in 'vlarb-scope' - any num of instances */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    cl_list_insert_tail(&p_current_vlarb_scope->across_list,tmp_str);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 2901 "osm_qos_parser_y.c"
break;
case 163:
#line 1430 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2908 "osm_qos_parser_y.c"
break;
case 164:
#line 1435 "osm_qos_parser_y.y"
	{
                            /* 'vl-high-limit' in 'vlarb-scope' - one instance of one number */
                            cl_list_iterator_t    list_iterator;
                            uint64_t            * p_tmp_num;

                            list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                            p_tmp_num = (uint64_t*)cl_list_obj(list_iterator);
                            if (p_tmp_num)
                            {
                                p_current_vlarb_scope->vl_high_limit = (uint32_t)(*p_tmp_num);
                                p_current_vlarb_scope->vl_high_limit_set = TRUE;
                                free(p_tmp_num);
                            }

                            cl_list_remove_all(&tmp_parser_struct.num_list);
                        }
#line 2928 "osm_qos_parser_y.c"
break;
case 165:
#line 1453 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2935 "osm_qos_parser_y.c"
break;
case 166:
#line 1458 "osm_qos_parser_y.y"
	{
                            /* 'vlarb-high' in 'vlarb-scope' - list of pairs of numbers with ':' and ',' */
                            cl_list_iterator_t    list_iterator;
                            uint64_t            * num_pair;

                            list_iterator = cl_list_head(&tmp_parser_struct.num_pair_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.num_pair_list) )
                            {
                                num_pair = (uint64_t*)cl_list_obj(list_iterator);
                                if (num_pair)
                                    cl_list_insert_tail(&p_current_vlarb_scope->vlarb_high_list,num_pair);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.num_pair_list);
                        }
#line 2954 "osm_qos_parser_y.c"
break;
case 167:
#line 1475 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2961 "osm_qos_parser_y.c"
break;
case 168:
#line 1480 "osm_qos_parser_y.y"
	{
                            /* 'vlarb-low' in 'vlarb-scope' - list of pairs of numbers with ':' and ',' */
                            cl_list_iterator_t    list_iterator;
                            uint64_t            * num_pair;

                            list_iterator = cl_list_head(&tmp_parser_struct.num_pair_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.num_pair_list) )
                            {
                                num_pair = (uint64_t*)cl_list_obj(list_iterator);
                                if (num_pair)
                                    cl_list_insert_tail(&p_current_vlarb_scope->vlarb_low_list,num_pair);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.num_pair_list);
                        }
#line 2980 "osm_qos_parser_y.c"
break;
case 169:
#line 1497 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 2987 "osm_qos_parser_y.c"
break;
case 170:
#line 1513 "osm_qos_parser_y.y"
	{
                            /* 'group' in 'sl2vl-scope' - any num of instances */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    cl_list_insert_tail(&p_current_sl2vl_scope->group_list,tmp_str);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3006 "osm_qos_parser_y.c"
break;
case 171:
#line 1530 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3013 "osm_qos_parser_y.c"
break;
case 172:
#line 1535 "osm_qos_parser_y.y"
	{
                            /* 'across' in 'sl2vl-scope' - any num of instances */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str) {
                                    cl_list_insert_tail(&p_current_sl2vl_scope->across_from_list,tmp_str);
                                    cl_list_insert_tail(&p_current_sl2vl_scope->across_to_list,strdup(tmp_str));
                                }
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3034 "osm_qos_parser_y.c"
break;
case 173:
#line 1554 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3041 "osm_qos_parser_y.c"
break;
case 174:
#line 1559 "osm_qos_parser_y.y"
	{
                            /* 'across-from' in 'sl2vl-scope' - any num of instances */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    cl_list_insert_tail(&p_current_sl2vl_scope->across_from_list,tmp_str);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3060 "osm_qos_parser_y.c"
break;
case 175:
#line 1576 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3067 "osm_qos_parser_y.c"
break;
case 176:
#line 1581 "osm_qos_parser_y.y"
	{
                            /* 'across-to' in 'sl2vl-scope' - any num of instances */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str) {
                                    cl_list_insert_tail(&p_current_sl2vl_scope->across_to_list,tmp_str);
                                }
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3087 "osm_qos_parser_y.c"
break;
case 177:
#line 1599 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3094 "osm_qos_parser_y.c"
break;
case 178:
#line 1604 "osm_qos_parser_y.y"
	{
                            /* 'from' in 'sl2vl-scope' - any num of instances */
                        }
#line 3101 "osm_qos_parser_y.c"
break;
case 179:
#line 1609 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3108 "osm_qos_parser_y.c"
break;
case 180:
#line 1614 "osm_qos_parser_y.y"
	{
                            /* 'to' in 'sl2vl-scope' - any num of instances */
                        }
#line 3115 "osm_qos_parser_y.c"
break;
case 181:
#line 1619 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3122 "osm_qos_parser_y.c"
break;
case 184:
#line 1628 "osm_qos_parser_y.y"
	{
                            int i;
                            for (i = 0; i < OSM_QOS_POLICY_MAX_PORTS_ON_SWITCH; i++)
                                p_current_sl2vl_scope->from[i] = TRUE;
                        }
#line 3131 "osm_qos_parser_y.c"
break;
case 187:
#line 1639 "osm_qos_parser_y.y"
	{
                            int i;
                            for (i = 0; i < OSM_QOS_POLICY_MAX_PORTS_ON_SWITCH; i++)
                                p_current_sl2vl_scope->to[i] = TRUE;
                        }
#line 3140 "osm_qos_parser_y.c"
break;
case 188:
#line 1646 "osm_qos_parser_y.y"
	{
                            int i;
                            cl_list_iterator_t    list_iterator;
                            uint64_t            * num_pair;
                            uint8_t               num1, num2;

                            list_iterator = cl_list_head(&tmp_parser_struct.num_pair_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.num_pair_list) )
                            {
                                num_pair = (uint64_t*)cl_list_obj(list_iterator);
                                if (num_pair)
                                {
                                    if ( num_pair[1] >= OSM_QOS_POLICY_MAX_PORTS_ON_SWITCH )
                                    {
                                        yyerror("port number out of range 'from' list");
                                        free(num_pair);
                                        cl_list_remove_all(&tmp_parser_struct.num_pair_list);
                                        return 1;
                                    }
                                    num1 = (uint8_t)num_pair[0];
                                    num2 = (uint8_t)num_pair[1];
                                    free(num_pair);
                                    for (i = num1; i <= num2; i++)
                                        p_current_sl2vl_scope->from[i] = TRUE;
                                }
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.num_pair_list);
                        }
#line 3173 "osm_qos_parser_y.c"
break;
case 189:
#line 1677 "osm_qos_parser_y.y"
	{
                            int i;
                            cl_list_iterator_t    list_iterator;
                            uint64_t            * num_pair;
                            uint8_t               num1, num2;

                            list_iterator = cl_list_head(&tmp_parser_struct.num_pair_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.num_pair_list) )
                            {
                                num_pair = (uint64_t*)cl_list_obj(list_iterator);
                                if (num_pair)
                                {
                                    if ( num_pair[1] >= OSM_QOS_POLICY_MAX_PORTS_ON_SWITCH )
                                    {
                                        yyerror("port number out of range 'to' list");
                                        free(num_pair);
                                        cl_list_remove_all(&tmp_parser_struct.num_pair_list);
                                        return 1;
                                    }
                                    num1 = (uint8_t)num_pair[0];
                                    num2 = (uint8_t)num_pair[1];
                                    free(num_pair);
                                    for (i = num1; i <= num2; i++)
                                        p_current_sl2vl_scope->to[i] = TRUE;
                                }
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.num_pair_list);
                        }
#line 3206 "osm_qos_parser_y.c"
break;
case 190:
#line 1709 "osm_qos_parser_y.y"
	{
                            /* 'sl2vl-table' - one instance of exactly
                               OSM_QOS_POLICY_SL2VL_TABLE_LEN numbers */
                            cl_list_iterator_t    list_iterator;
                            uint64_t              num;
                            uint64_t            * p_num;
                            int                   i = 0;

                            if (p_current_sl2vl_scope->sl2vl_table_set)
                            {
                                yyerror("sl2vl-scope has more than one sl2vl-table");
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }

                            if (cl_list_count(&tmp_parser_struct.num_list) != OSM_QOS_POLICY_SL2VL_TABLE_LEN)
                            {
                                yyerror("wrong number of values in 'sl2vl-table' (should be 16)");
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }

                            list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.num_list) )
                            {
                                p_num = (uint64_t*)cl_list_obj(list_iterator);
                                num = *p_num;
                                free(p_num);
                                if (num >= OSM_QOS_POLICY_MAX_VL_NUM)
                                {
                                    yyerror("wrong VL value in 'sl2vl-table' (should be 0 to 15)");
                                    cl_list_remove_all(&tmp_parser_struct.num_list);
                                    return 1;
                                }

                                p_current_sl2vl_scope->sl2vl_table[i++] = (uint8_t)num;
                                list_iterator = cl_list_next(list_iterator);
                            }
                            p_current_sl2vl_scope->sl2vl_table_set = TRUE;
                            cl_list_remove_all(&tmp_parser_struct.num_list);
                        }
#line 3251 "osm_qos_parser_y.c"
break;
case 191:
#line 1752 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3258 "osm_qos_parser_y.c"
break;
case 192:
#line 1769 "osm_qos_parser_y.y"
	{
                            /* 'name' of 'qos-level' - one instance */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            if (p_current_qos_level->name)
                            {
                                yyerror("qos-level has multiple 'name' tags");
                                cl_list_remove_all(&tmp_parser_struct.str_list);
                                return 1;
                            }

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            if ( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    p_current_qos_level->name = tmp_str;
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3283 "osm_qos_parser_y.c"
break;
case 193:
#line 1792 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3290 "osm_qos_parser_y.c"
break;
case 194:
#line 1797 "osm_qos_parser_y.y"
	{
                            /* 'use' of 'qos-level' - one instance */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            if (p_current_qos_level->use)
                            {
                                yyerror("qos-level has multiple 'use' tags");
                                cl_list_remove_all(&tmp_parser_struct.str_list);
                                return 1;
                            }

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            if ( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    p_current_qos_level->use = tmp_str;
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3315 "osm_qos_parser_y.c"
break;
case 195:
#line 1820 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3322 "osm_qos_parser_y.c"
break;
case 196:
#line 1825 "osm_qos_parser_y.y"
	{
                            /* 'sl' in 'qos-level' - one instance */
                            cl_list_iterator_t   list_iterator;
                            uint64_t           * p_num;

                            if (p_current_qos_level->sl_set)
                            {
                                yyerror("'qos-level' has multiple 'sl' tags");
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }
                            list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                            p_num = (uint64_t*)cl_list_obj(list_iterator);
                            p_current_qos_level->sl = (uint8_t)(*p_num);
                            free(p_num);
                            p_current_qos_level->sl_set = TRUE;
                            cl_list_remove_all(&tmp_parser_struct.num_list);
                        }
#line 3344 "osm_qos_parser_y.c"
break;
case 197:
#line 1845 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3351 "osm_qos_parser_y.c"
break;
case 198:
#line 1850 "osm_qos_parser_y.y"
	{
                            /* 'mtu-limit' in 'qos-level' - one instance */
                            cl_list_iterator_t   list_iterator;
                            uint64_t           * p_num;

                            if (p_current_qos_level->mtu_limit_set)
                            {
                                yyerror("'qos-level' has multiple 'mtu-limit' tags");
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }
                            list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                            p_num = (uint64_t*)cl_list_obj(list_iterator);
                            if (*p_num > OSM_QOS_POLICY_MAX_MTU || *p_num < OSM_QOS_POLICY_MIN_MTU)
                            {
                                yyerror("mtu limit is out of range, value: %d", *p_num);
                                free(p_num);
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }
                            p_current_qos_level->mtu_limit = (uint8_t)(*p_num);
                            free(p_num);
                            p_current_qos_level->mtu_limit_set = TRUE;
                            cl_list_remove_all(&tmp_parser_struct.num_list);
                        }
#line 3380 "osm_qos_parser_y.c"
break;
case 199:
#line 1877 "osm_qos_parser_y.y"
	{
                            /* 'mtu-limit' in 'qos-level' - one instance */
                            RESET_BUFFER;
                        }
#line 3388 "osm_qos_parser_y.c"
break;
case 200:
#line 1883 "osm_qos_parser_y.y"
	{
                            /* 'rate-limit' in 'qos-level' - one instance */
                            cl_list_iterator_t   list_iterator;
                            uint64_t           * p_num;

                            if (p_current_qos_level->rate_limit_set)
                            {
                                yyerror("'qos-level' has multiple 'rate-limit' tags");
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }
                            list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                            p_num = (uint64_t*)cl_list_obj(list_iterator);
                            if (*p_num > OSM_QOS_POLICY_MAX_RATE || *p_num < OSM_QOS_POLICY_MIN_RATE)
                            {
                                yyerror("rate limit is out of range, value: %d", *p_num);
                                free(p_num);
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }
                            p_current_qos_level->rate_limit = (uint8_t)(*p_num);
                            free(p_num);
                            p_current_qos_level->rate_limit_set = TRUE;
                            cl_list_remove_all(&tmp_parser_struct.num_list);
                        }
#line 3417 "osm_qos_parser_y.c"
break;
case 201:
#line 1910 "osm_qos_parser_y.y"
	{
                            /* 'rate-limit' in 'qos-level' - one instance */
                            RESET_BUFFER;
                        }
#line 3425 "osm_qos_parser_y.c"
break;
case 202:
#line 1916 "osm_qos_parser_y.y"
	{
                            /* 'packet-life' in 'qos-level' - one instance */
                            cl_list_iterator_t   list_iterator;
                            uint64_t           * p_num;

                            if (p_current_qos_level->pkt_life_set)
                            {
                                yyerror("'qos-level' has multiple 'packet-life' tags");
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }
                            list_iterator = cl_list_head(&tmp_parser_struct.num_list);
                            p_num = (uint64_t*)cl_list_obj(list_iterator);
                            p_current_qos_level->pkt_life = (uint8_t)(*p_num);
                            free(p_num);
                            p_current_qos_level->pkt_life_set= TRUE;
                            cl_list_remove_all(&tmp_parser_struct.num_list);
                        }
#line 3447 "osm_qos_parser_y.c"
break;
case 203:
#line 1936 "osm_qos_parser_y.y"
	{
                            /* 'packet-life' in 'qos-level' - one instance */
                            RESET_BUFFER;
                        }
#line 3455 "osm_qos_parser_y.c"
break;
case 204:
#line 1942 "osm_qos_parser_y.y"
	{
                            /* 'path-bits' in 'qos-level' - any num of instances */
                            /* list of path bit ranges */

                            if (cl_list_count(&tmp_parser_struct.num_pair_list))
                            {
                                uint64_t ** range_arr;
                                unsigned range_len;

                                __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                                      &range_arr,
                                                      &range_len );

                                if ( !p_current_qos_level->path_bits_range_len )
                                {
                                    p_current_qos_level->path_bits_range_arr = range_arr;
                                    p_current_qos_level->path_bits_range_len = range_len;
                                }
                                else
                                {
                                    uint64_t ** new_range_arr;
                                    unsigned new_range_len;
                                    __merge_rangearr( p_current_qos_level->path_bits_range_arr,
                                                      p_current_qos_level->path_bits_range_len,
                                                      range_arr,
                                                      range_len,
                                                      &new_range_arr,
                                                      &new_range_len );
                                    p_current_qos_level->path_bits_range_arr = new_range_arr;
                                    p_current_qos_level->path_bits_range_len = new_range_len;
                                }
                            }
                        }
#line 3492 "osm_qos_parser_y.c"
break;
case 205:
#line 1977 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3499 "osm_qos_parser_y.c"
break;
case 206:
#line 1982 "osm_qos_parser_y.y"
	{
                            /* 'pkey' in 'qos-level' - num of instances of list of ranges */
                            if (cl_list_count(&tmp_parser_struct.num_pair_list))
                            {
                                uint64_t ** range_arr;
                                unsigned range_len;

                                __pkey_rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                                      &range_arr,
                                                      &range_len );

                                if ( !p_current_qos_level->pkey_range_len )
                                {
                                    p_current_qos_level->pkey_range_arr = range_arr;
                                    p_current_qos_level->pkey_range_len = range_len;
                                }
                                else
                                {
                                    uint64_t ** new_range_arr;
                                    unsigned new_range_len;
                                    __merge_rangearr( p_current_qos_level->pkey_range_arr,
                                                      p_current_qos_level->pkey_range_len,
                                                      range_arr,
                                                      range_len,
                                                      &new_range_arr,
                                                      &new_range_len );
                                    p_current_qos_level->pkey_range_arr = new_range_arr;
                                    p_current_qos_level->pkey_range_len = new_range_len;
                                }
                            }
                        }
#line 3534 "osm_qos_parser_y.c"
break;
case 207:
#line 2015 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3541 "osm_qos_parser_y.c"
break;
case 208:
#line 2032 "osm_qos_parser_y.y"
	{
                            /* 'use' of 'qos-match-rule' - one instance */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            if (p_current_qos_match_rule->use)
                            {
                                yyerror("'qos-match-rule' has multiple 'use' tags");
                                cl_list_remove_all(&tmp_parser_struct.str_list);
                                return 1;
                            }

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            if ( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    p_current_qos_match_rule->use = tmp_str;
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3566 "osm_qos_parser_y.c"
break;
case 209:
#line 2055 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3573 "osm_qos_parser_y.c"
break;
case 210:
#line 2060 "osm_qos_parser_y.y"
	{
                            /* 'qos-class' in 'qos-match-rule' - num of instances of list of ranges */
                            /* list of class ranges (QoS Class is 12-bit value) */
                            if (cl_list_count(&tmp_parser_struct.num_pair_list))
                            {
                                uint64_t ** range_arr;
                                unsigned range_len;

                                __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                                      &range_arr,
                                                      &range_len );

                                if ( !p_current_qos_match_rule->qos_class_range_len )
                                {
                                    p_current_qos_match_rule->qos_class_range_arr = range_arr;
                                    p_current_qos_match_rule->qos_class_range_len = range_len;
                                }
                                else
                                {
                                    uint64_t ** new_range_arr;
                                    unsigned new_range_len;
                                    __merge_rangearr( p_current_qos_match_rule->qos_class_range_arr,
                                                      p_current_qos_match_rule->qos_class_range_len,
                                                      range_arr,
                                                      range_len,
                                                      &new_range_arr,
                                                      &new_range_len );
                                    p_current_qos_match_rule->qos_class_range_arr = new_range_arr;
                                    p_current_qos_match_rule->qos_class_range_len = new_range_len;
                                }
                            }
                        }
#line 3609 "osm_qos_parser_y.c"
break;
case 211:
#line 2094 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3616 "osm_qos_parser_y.c"
break;
case 212:
#line 2099 "osm_qos_parser_y.y"
	{
                            /* 'source' in 'qos-match-rule' - text */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    cl_list_insert_tail(&p_current_qos_match_rule->source_list,tmp_str);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3635 "osm_qos_parser_y.c"
break;
case 213:
#line 2116 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3642 "osm_qos_parser_y.c"
break;
case 214:
#line 2121 "osm_qos_parser_y.y"
	{
                            /* 'destination' in 'qos-match-rule' - text */
                            cl_list_iterator_t    list_iterator;
                            char                * tmp_str;

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            while( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    cl_list_insert_tail(&p_current_qos_match_rule->destination_list,tmp_str);
                                list_iterator = cl_list_next(list_iterator);
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3661 "osm_qos_parser_y.c"
break;
case 215:
#line 2138 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3668 "osm_qos_parser_y.c"
break;
case 216:
#line 2143 "osm_qos_parser_y.y"
	{
                            /* 'qos-level-name' in 'qos-match-rule' - single string */
                            cl_list_iterator_t   list_iterator;
                            char               * tmp_str;

                            if (p_current_qos_match_rule->qos_level_name)
                            {
                                yyerror("qos-match-rule has multiple 'qos-level-name' tags");
                                cl_list_remove_all(&tmp_parser_struct.num_list);
                                return 1;
                            }

                            list_iterator = cl_list_head(&tmp_parser_struct.str_list);
                            if ( list_iterator != cl_list_end(&tmp_parser_struct.str_list) )
                            {
                                tmp_str = (char*)cl_list_obj(list_iterator);
                                if (tmp_str)
                                    p_current_qos_match_rule->qos_level_name = tmp_str;
                            }
                            cl_list_remove_all(&tmp_parser_struct.str_list);
                        }
#line 3693 "osm_qos_parser_y.c"
break;
case 217:
#line 2166 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3700 "osm_qos_parser_y.c"
break;
case 218:
#line 2171 "osm_qos_parser_y.y"
	{
                            /* 'service-id' in 'qos-match-rule' - num of instances of list of ranges */
                            if (cl_list_count(&tmp_parser_struct.num_pair_list))
                            {
                                uint64_t ** range_arr;
                                unsigned range_len;

                                __rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                                      &range_arr,
                                                      &range_len );

                                if ( !p_current_qos_match_rule->service_id_range_len )
                                {
                                    p_current_qos_match_rule->service_id_range_arr = range_arr;
                                    p_current_qos_match_rule->service_id_range_len = range_len;
                                }
                                else
                                {
                                    uint64_t ** new_range_arr;
                                    unsigned new_range_len;
                                    __merge_rangearr( p_current_qos_match_rule->service_id_range_arr,
                                                      p_current_qos_match_rule->service_id_range_len,
                                                      range_arr,
                                                      range_len,
                                                      &new_range_arr,
                                                      &new_range_len );
                                    p_current_qos_match_rule->service_id_range_arr = new_range_arr;
                                    p_current_qos_match_rule->service_id_range_len = new_range_len;
                                }
                            }
                        }
#line 3735 "osm_qos_parser_y.c"
break;
case 219:
#line 2204 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3742 "osm_qos_parser_y.c"
break;
case 220:
#line 2209 "osm_qos_parser_y.y"
	{
                            /* 'pkey' in 'qos-match-rule' - num of instances of list of ranges */
                            if (cl_list_count(&tmp_parser_struct.num_pair_list))
                            {
                                uint64_t ** range_arr;
                                unsigned range_len;

                                __pkey_rangelist2rangearr( &tmp_parser_struct.num_pair_list,
                                                      &range_arr,
                                                      &range_len );

                                if ( !p_current_qos_match_rule->pkey_range_len )
                                {
                                    p_current_qos_match_rule->pkey_range_arr = range_arr;
                                    p_current_qos_match_rule->pkey_range_len = range_len;
                                }
                                else
                                {
                                    uint64_t ** new_range_arr;
                                    unsigned new_range_len;
                                    __merge_rangearr( p_current_qos_match_rule->pkey_range_arr,
                                                      p_current_qos_match_rule->pkey_range_len,
                                                      range_arr,
                                                      range_len,
                                                      &new_range_arr,
                                                      &new_range_len );
                                    p_current_qos_match_rule->pkey_range_arr = new_range_arr;
                                    p_current_qos_match_rule->pkey_range_len = new_range_len;
                                }
                            }
                        }
#line 3777 "osm_qos_parser_y.c"
break;
case 221:
#line 2242 "osm_qos_parser_y.y"
	{
                            RESET_BUFFER;
                        }
#line 3784 "osm_qos_parser_y.c"
break;
case 222:
#line 2253 "osm_qos_parser_y.y"
	{
                        cl_list_insert_tail(&tmp_parser_struct.str_list,
                                            strdup(__parser_strip_white(tmp_parser_struct.str)));
                        tmp_parser_struct.str[0] = '\0';
                    }
#line 3793 "osm_qos_parser_y.c"
break;
case 225:
#line 2264 "osm_qos_parser_y.y"
	{
                        strcat(tmp_parser_struct.str,yystack.l_mark[0]);
                        free(yystack.l_mark[0]);
                    }
#line 3801 "osm_qos_parser_y.c"
break;
case 231:
#line 2284 "osm_qos_parser_y.y"
	{
                        uint64_t * p_num = (uint64_t*)malloc(sizeof(uint64_t));
                        __parser_str2uint64(p_num,yystack.l_mark[0]);
                        free(yystack.l_mark[0]);
                        cl_list_insert_tail(&tmp_parser_struct.num_list, p_num);
                    }
#line 3811 "osm_qos_parser_y.c"
break;
case 232:
#line 2292 "osm_qos_parser_y.y"
	{
                        uint64_t * num_pair = (uint64_t*)malloc(sizeof(uint64_t)*2);
                        num_pair[0] = tmp_parser_struct.num_pair[0];
                        num_pair[1] = tmp_parser_struct.num_pair[1];
                        cl_list_insert_tail(&tmp_parser_struct.num_pair_list, num_pair);
                    }
#line 3821 "osm_qos_parser_y.c"
break;
case 233:
#line 2298 "osm_qos_parser_y.y"
	{
                        uint64_t * num_pair = (uint64_t*)malloc(sizeof(uint64_t)*2);
                        num_pair[0] = tmp_parser_struct.num_pair[0];
                        num_pair[1] = tmp_parser_struct.num_pair[1];
                        cl_list_insert_tail(&tmp_parser_struct.num_pair_list, num_pair);
                    }
#line 3831 "osm_qos_parser_y.c"
break;
case 234:
#line 2306 "osm_qos_parser_y.y"
	{
                        __parser_str2uint64(&tmp_parser_struct.num_pair[0],yystack.l_mark[0]);
                        free(yystack.l_mark[0]);
                    }
#line 3839 "osm_qos_parser_y.c"
break;
case 235:
#line 2312 "osm_qos_parser_y.y"
	{
                        __parser_str2uint64(&tmp_parser_struct.num_pair[1],yystack.l_mark[0]);
                        free(yystack.l_mark[0]);
                    }
#line 3847 "osm_qos_parser_y.c"
break;
case 237:
#line 2321 "osm_qos_parser_y.y"
	{
                        uint64_t * num_pair = (uint64_t*)malloc(sizeof(uint64_t)*2);
                        num_pair[0] = tmp_parser_struct.num_pair[0];
                        num_pair[1] = tmp_parser_struct.num_pair[1];
                        cl_list_insert_tail(&tmp_parser_struct.num_pair_list, num_pair);
                    }
#line 3857 "osm_qos_parser_y.c"
break;
case 238:
#line 2327 "osm_qos_parser_y.y"
	{
                        uint64_t * num_pair = (uint64_t*)malloc(sizeof(uint64_t)*2);
                        if (tmp_parser_struct.num_pair[0] <= tmp_parser_struct.num_pair[1]) {
                            num_pair[0] = tmp_parser_struct.num_pair[0];
                            num_pair[1] = tmp_parser_struct.num_pair[1];
                        }
                        else {
                            num_pair[1] = tmp_parser_struct.num_pair[0];
                            num_pair[0] = tmp_parser_struct.num_pair[1];
                        }
                        cl_list_insert_tail(&tmp_parser_struct.num_pair_list, num_pair);
                    }
#line 3873 "osm_qos_parser_y.c"
break;
case 239:
#line 2339 "osm_qos_parser_y.y"
	{
                        uint64_t * num_pair = (uint64_t*)malloc(sizeof(uint64_t)*2);
                        if (tmp_parser_struct.num_pair[0] <= tmp_parser_struct.num_pair[1]) {
                            num_pair[0] = tmp_parser_struct.num_pair[0];
                            num_pair[1] = tmp_parser_struct.num_pair[1];
                        }
                        else {
                            num_pair[1] = tmp_parser_struct.num_pair[0];
                            num_pair[0] = tmp_parser_struct.num_pair[1];
                        }
                        cl_list_insert_tail(&tmp_parser_struct.num_pair_list, num_pair);
                    }
#line 3889 "osm_qos_parser_y.c"
break;
case 240:
#line 2351 "osm_qos_parser_y.y"
	{
                        uint64_t * num_pair = (uint64_t*)malloc(sizeof(uint64_t)*2);
                        num_pair[0] = tmp_parser_struct.num_pair[0];
                        num_pair[1] = tmp_parser_struct.num_pair[1];
                        cl_list_insert_tail(&tmp_parser_struct.num_pair_list, num_pair);
                    }
#line 3899 "osm_qos_parser_y.c"
break;
case 241:
#line 2359 "osm_qos_parser_y.y"
	{
                        __parser_str2uint64(&tmp_parser_struct.num_pair[0],yystack.l_mark[0]);
                        __parser_str2uint64(&tmp_parser_struct.num_pair[1],yystack.l_mark[0]);
                        free(yystack.l_mark[0]);
                    }
#line 3908 "osm_qos_parser_y.c"
break;
case 242:
#line 2366 "osm_qos_parser_y.y"
	{
                        __parser_str2uint64(&tmp_parser_struct.num_pair[0],yystack.l_mark[0]);
                        free(yystack.l_mark[0]);
                    }
#line 3916 "osm_qos_parser_y.c"
break;
case 243:
#line 2372 "osm_qos_parser_y.y"
	{
                        __parser_str2uint64(&tmp_parser_struct.num_pair[1],yystack.l_mark[0]);
                        free(yystack.l_mark[0]);
                    }
#line 3924 "osm_qos_parser_y.c"
break;
#line 3926 "osm_qos_parser_y.c"
    }
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yystack.s_mark = YYFINAL;
        *++yystack.l_mark = yyval;
        if (yychar < 0)
        {
            yychar = YYLEX;
            if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
            if (yydebug)
            {
                if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == YYEOF) goto yyaccept;
        goto yyloop;
    }
    if (((yyn = yygindex[yym]) != 0) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yystack.s_mark, yystate);
#endif
    if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    *++yystack.s_mark = (YYINT) yystate;
    *++yystack.l_mark = yyval;
    goto yyloop;

yyoverflow:
    YYERROR_CALL("yacc stack overflow");

yyabort:
    yyfreestack(&yystack);
    return (1);

yyaccept:
    yyfreestack(&yystack);
    return (0);
}
